/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.graphics.compact.CompactRenderingViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date");
dojo.require("rmpw.panner.PannableLayer");
dojo.require("rmps.core.commenting.CommentClient");
dojo.require("rmps.graphics.compact.CompactRenderingDiagram");
dojo.require("jazz.app.context");
dojo.require("jazz.util.iframe");
dojo.require("jazz.ui.ResourceLink");

dojo.requireLocalization("rmps.graphics.compact", "CompactRenderingViewerMessages");
dojo.requireLocalization("rmps.app.ui.internal.viewers", "ResourceMessages");

var ResourceLink = jazz.ui.ResourceLink;

dojo.declare("rmps.graphics.compact.CompactRenderingViewer", [dijit._Widget, dijit._Templated], {
	templateString: dojo.cache("rmps.app", "ui/internal/viewers/templates/Resource.html"),
	
	crvMessages: dojo.i18n.getLocalization("rmps.graphics.compact", "CompactRenderingViewerMessages"),
	messages: dojo.i18n.getLocalization("rmps.app.ui.internal.viewers", "ResourceMessages"),
	
	
	containerDiagram: null, // contain the image diagram
	editorInput: null,
	
	_diagram: null,
	_defaultImage: dojo.moduleUrl("dojox.widget.Loader","icons/loading.gif"),
	
	postCreate: function() {
		this.inherited(arguments);
	},
	
	 /** 
	  * FIXME:
	  * This method is a temp way to get the last comment made until we have one in the server side.
	  * This oepration should not be handled in the viewer class*/	
	_loadComments: function(comments){
		try {
			if(comments.length){
				/*We are going to loop through comments to get the last comment made 
				 * We will asusme that the first comment in the array is the first one. Then, compare each
				 * comment to see if it has been added/modified after the current assumed last comment
				 * */
				var lastCommentTimestamp= comments[0].created;	/*I will assume that created can never be null in order to avoid extra checks */
				for (var i = 1; i < comments.length; i++) {
					var curCommentTimestamp= comments[i].created; 
					if(dojo.date.compare(curCommentTimestamp, lastCommentTimestamp)>0){
						lastCommentTimestamp= curCommentTimestamp; /*Replace the assumed value*/
					}
				}
				var date = dojo.date.locale.format(lastCommentTimestamp, {formatLength: "medium"});
				this.commentNode.appendChild(dojo.doc.createTextNode(" " + date));	
			}else {
				this.commentNode.appendChild(dojo.doc.createTextNode(" " + this.crvMessages.noComments));
				this.commentNode.style.fontStyle= "italic";	/*It should be italic, as it is an info message */
			}	
		} catch (e) {
			console.debug(e);
		}
		
		
	},
	
	/** @see rmps.graphics.viewer.model.CompactRenderingViewer#_loadComments*/
	setCommentNode: function(){
		var deferred = rmps.core.commenting.CommentClient.getCommentsFor(this.editorInput.getResourceUri(), jazz.app.context.get().baseline);
		deferred.addCallback(this, this._loadComments);
		return deferred;		
	},

	startup: function() {
		this.inherited(arguments);
		var isConfig = this.editorInput.getResourceUri().indexOf('/cm/') !== -1 || this.editorInput.getResourceUri().indexOf('/oslc_config/') !== -1
		var projectArea = this.editorInput.getProjectArea();
		if (projectArea) {
			this.projectNode.appendChild(dojo.doc.createTextNode(" " + projectArea));			
		} else {
			dojo.style(this.projectRow, "display", "none");
		}
		var longName = " " + this.editorInput.getLongName(); // long name
		if (isConfig) {
			this.nameNode.appendChild(dojo.doc.createTextNode(longName));
			dojo.style(this.longNameRow, "display", "none");
			dojo.style(this.nameRow, "display", "");
			dojo.style(this.configurationRow, "display", "none");
		} else {
			/* 
			 * When user put the mouse on link, a new float window is showing up.
			 * This part is handling the widnow frame size.  
			*/
			long_name_width = "195px"; // default
			parent_frame = window.frameElement;
			if(parent_frame) {
				long_name_width = parent_frame.style.width; // change width to be the same as frame width
				long_name_width = long_name_width.substring(0, long_name_width.length - 2); // remove 'px'
				long_name_width = (parseInt(long_name_width)-100).toString()+"px";
			}
			
			longName = longName.replace(new RegExp('::', 'g'), ':: ');
			this.longNameNode.appendChild(dojo.doc.createTextNode(longName));
			dojo.style(this.longNameNode, "float", "left");
			dojo.style(this.longNameNode, "width", long_name_width);
			dojo.style(this.longNameNode, "word-wrap", "break-word"); // add word-wrap to force break line (for realy long names)
			var configuration = this.editorInput.getConfiguration();
			var configurationName = this.editorInput.getConfigurationName();
			if (configuration && configurationName) {
				var rl = new ResourceLink({uri: configuration, label: configurationName});
				this.configurationNode.appendChild(rl.domNode);				
			} else {
				dojo.style(this.configurationRow, "display", "none");				
			}
		}
		this.typeNode.appendChild(dojo.doc.createTextNode(" " + this.editorInput.getDisplayType()));
		this.modifiedNode.appendChild(dojo.doc.createTextNode(" " + this.editorInput.getModificationTime()));
		if (isConfig) {
			dojo.style(this.commentRow, "display", "none");
		} else {
			this.setCommentNode();			
		}
		
		if(this.editorInput.getIsDiagram()){
			if(this.editorInput.getIsExpanded()){
				
				/*we do not show thumnail in the large preview */
				this.thumbnailArea.style.display= "none";
				this.diagramNode.style.display= "block";
				
				/*No need for our case to have layer; therefore, we are calling the diagram directly through the
				 * viewer */
				this._diagram= new rmps.graphics.compact.CompactRenderingDiagram(this.diagramNode, this.containerNode,
												this.editorInput.getImageUri(), this.editorInput.getConfiguration());
			}else {
				this.thumbnailArea.src= this.editorInput.getImageUri();
				this.thumbnailArea.height= "100";
				this.thumbnailArea.width= "100";
				this.diagramNode.style.display= "none";
			}
		}else {
				this.thumbnailArea.style.display= "none";
				this.diagramNode.style.display= "none";
		}

		this.containerDiagram.appendChild(this.containerNode);
		
		if (window.postMessage && window.parent) {
			var newPreviewHeight = jazz.util.iframe.getDocumentHeight();
			var resizeMessage = "oslc-preview-height:" + newPreviewHeight;
			window.parent.postMessage(resizeMessage, "*");
			resizeMessage = "jazz-compact-rendering-preview-height:" + newPreviewHeight;
			window.parent.postMessage(resizeMessage, "*");
		} 
		
		this.resize();
	},
	
	resize: function(size) {
		/*
		 * The resize is handled by the diagram. 
		 * 
		 * Empty block 
		 * */
	}
});

}());
