/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

(function() {

dojo.provide("rmps.app.pickers.internal.OslcConfigurationPicker");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.i18n");
dojo.require("dojo.store.Memory");
dojo.require("dojo.NodeList-traverse");
dojo.require("dojox.html.ellipsis");

dojo.require("com.ibm.team.gc.sdk.web.resources.ScopeMenuButton");
dojo.require("com.ibm.team.gc.sdk.web.resources.ScrollableTable");

dojo.require("com.ibm.team.configuration.web.context.ContextAPI");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders"); // Workaround to get all provides necessary for above
dojo.require("dojo.DeferredList"); // Workaround to get all provides necessary for above

dojo.require('jazz.app.auth');
dojo.require("jazz.ui.RemoteChooserResponder");
dojo.require("jazz.ui.PaginationControl");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.ui.Select");
dojo.require("jazz.ui.toolbar.TwoStateButton");

dojo.require("rmps.app.pickers.internal.LiveSearchBox");

dojo.require("rmps.core.projects.ProjectClient");
dojo.require("rmps.core.oslc_config.OslcConfigClient");
dojo.require("rmps.core.util.DateUtils");

dojo.require("rmps.app.ui.internal.util.Constants");
dojo.require("com.ibm.team.configuration.web.context.ContextPicker");
var Constants = rmps.app.ui.internal.util.Constants;

dojo.requireLocalization("rmps.app.pickers.internal", "OslcConfigurationPickerMessages");
var messages = dojo.i18n.getLocalization("rmps.app.pickers.internal", "OslcConfigurationPickerMessages");

var PROJECT_NAME = "http://jazz.net/xmlns/rmps/groups/1.0/projectName";

var ProjectClient = rmps.core.projects.ProjectClient;
var OslcConfigClient = rmps.core.oslc_config.OslcConfigClient;
var DateUtils = rmps.core.util.DateUtils;
var ContextAPI = com.ibm.team.configuration.web.context.ContextAPI;
var ResourceLink = jazz.ui.ResourceLink;

var ContextPickerConfigurationFilterTypes = com.ibm.team.configuration.web.context.ContextPickerConfigurationFilterTypes;

dojo.declare("rmps.app.pickers.internal.OslcConfigurationPicker", [dijit._Widget, dijit._Templated], {

	PAGE_SIZE: 10,
	NUMBER_OF_RECENTS: 20,
	
	messages: messages,
	
	templateString: dojo.cache("rmps.app", "pickers/internal/templates/OslcConfigurationPicker.html"),
	widgetsInTemplate: true,

	searchLabel: '',
	searchTitle: '',

	_isModule: true, //set false if this module will be used as a regular widget
	_remoteFrame: null, // The remote frame to communicate with
	_initialModuleWidth: "650px",
	_initialModuleHeight: "464px",
	_initialModuleHeightWithProjectSelection: "498px",

	_configurationTable: null,
	
	_projectUri: null,
	_projectSelect: null,
	_currentScope: null,
	_paginationControl: null,
	
	_selection: null,
	
	_hasSearched: false,
	_recentConfigurations: null,
	
	_blankGif: dojo.moduleUrl("dojo", "resources/blank.gif").toString(),
	
	postCreate: function() {
		this.inherited(arguments);
			
		this.liveSearchBox.delegate = this;
		this.liveSearchBox.set('placeholder', this.messages.searchFieldText);
		
		// Check if projectareaUri has been passed.
		var searchString = location.search.replace(/^\?/, ''); // remove leading '?'
		var args = dojo.queryToObject(searchString);
		var componentUri = args.componentUri || args['jazz_config.component']; 
		if (componentUri) {
			this._getProjectFromComponentUrl(componentUri).then(dojo.hitch(this, function(result) {
				this._projectUri = result; 
				// Add project title
				this._addProjectTitle();
			}));
		} else if (args.projectareaUri && args.projectareaUri !== Constants.NONE) {
			// Hide project area section
			dojo.style(this._projectSection, "display", "none"); 
			this._projectUri = unescape(args.projectareaUri);
		} else {
			// Populate project selection
			this._populateProjects();
		}
		
		var scopeMenu = new com.ibm.team.gc.sdk.web.resources.ScopeMenuButton();
		var scopes = [];
		var filtered = null;
		if (args.filter) {
			var filterObject = dojo.fromJson(args.filter);	
			if (filterObject && filterObject.validTypes) {
				filtered = 	filterObject.validTypes;
			}	
		}

	    if (!filtered || filtered.indexOf(ContextPickerConfigurationFilterTypes.streams) >= 0)
			scopes.push({id:"stream", title:messages.Streams, label:messages.Streams, imgClass:'rmps-ui-resources-oslc-configuration-picker-stream-icon'});
	    if (!filtered || filtered.indexOf(ContextPickerConfigurationFilterTypes.baselines) >= 0)
			scopes.push({id:"baseline", title:messages.Snapshots, label:messages.Snapshots, imgClass:'rmps-ui-resources-oslc-configuration-picker-baseline-icon'});
	    if (!filtered || filtered.indexOf(ContextPickerConfigurationFilterTypes.changeSets) >= 0)
			scopes.push({id:"changeSet", title:messages.Changesets, label:messages.Changesets, imgClass:'rmps-ui-resources-oslc-configuration-picker-changeset-icon'});

		scopeMenu.set("scopes", scopes);
		this._currentScope = scopes[0];
		scopeMenu.set('currentScope', this._currentScope);
		scopeMenu.watch('currentScope', dojo.hitch(this, this._onScopeChanged));
		
		this._scopeMenuNode.appendChild(scopeMenu.domNode);
		scopeMenu.startup();
				
		this._configurationTable = new com.ibm.team.gc.sdk.web.resources.ScrollableTable({
			picker: this,
			rowFactory: dojo.hitch(this, this.rowFactory),
			columnFactory: dojo.hitch(this, this.columnFactory),
			initialSortByAscending: false
		});
		dojo.place(this._configurationTable.domNode, this._configTableNode, "only");
		// We have our own button for this
		this._configurationTable.hideAllFilterButtons();

		this.connect(this._configurationTable, 'onSelectionChanged', this._onSelectionChanged);
		this.connect(this._configurationTable, 'onDoubleClickItem', this._onDoubleClickItem);

		// Filter toggle button
		var filterToggleButton = new jazz.ui.toolbar.TwoStateButton({
			imgClass: "filter-toggle", 
			title: this.messages.filters,
			label: this.messages.filters,
			onClick: dojo.hitch(this, function(event) {
				this._configurationTable._tableHeader._onClickFilter(0, event);
			})
		});
		this._filterToggleNode.appendChild(filterToggleButton.domNode);
		
		this._paginationControl = new jazz.ui.PaginationControl({
			increment: this.PAGE_SIZE,
			onNextClick: dojo.hitch(this, function(p, start, end) {
				this._refresh(this.liveSearchBox.getSearchString(), Math.floor(start / this.PAGE_SIZE));
			}),
			onPreviousClick: dojo.hitch(this, function(p, start, end) {
				this._refresh(this.liveSearchBox.getSearchString(), Math.floor(start / this.PAGE_SIZE));
			})
		});
		this._paginationControlNode.appendChild(this._paginationControl.domNode);
		
		if (this._isModule){
			this._remoteFrame = new jazz.ui.RemoteChooserResponder();
			this.own(this._remoteFrame);

			//Send OSLC message to let the dialog containing this module know what size to make itself.
			if (args.projectareaUri && args.projectareaUri !== Constants.NONE) {
				this._remoteFrame.sendResize(this._initialModuleWidth, this._initialModuleHeight);				
			} else {
				this._remoteFrame.sendResize(this._initialModuleWidth, this._initialModuleHeightWithProjectSelection);
			}
		}
		
		// Start with getting recent configurations
		this._refresh();
	},
	
	startup: function() {
		this.inherited(arguments);

		this._configurationTable.startup();
	},
	
	_getProjectFromComponentUrl: function(componentUri) {
		return OslcConfigClient.getProjectUriFromComponentUri(componentUri);
	},
	
	_addProjectTitle: function() {
		this._projectNode.innerHTML = this.messages.loading;

		ProjectClient.getProjectName(this._projectUri).then(dojo.hitch(this, function(name) {
			this._projectNode.innerHTML = name;
		}));
	},
	
	_populateProjects: function() {
		this._projectNode.innerHTML = this.messages.loading;
		
		// Get all the projects and sort by name.
		var projectsDeferred = ProjectClient.getAllProjects(false, false);			
		projectsDeferred.then(dojo.hitch(this, function(projects) {
			var projectsData = [];
			for (var uri in projects) {
				if (!this.projectsFilter || this.projectsFilter(projects[uri])) {
					projectsData.push({
						id: uri,
						label: projects[uri][PROJECT_NAME][0].value
					});
				}
			}
			
			projectsData.sort(function(a, b) {return a.label < b.label ? -1 : a.label > b.label ? 1 : 0;}); 
			var store = new dojo.store.Memory({
				data: projectsData
			});

			// Create a project drop-down and associate it with the project store.
			this._projectSelect = new dijit.form.FilteringSelect({
				store: store,
				searchAttr: "label",
				required: false,
				selectOnClick: false,
				onChange: dojo.hitch(this, this._projectChanged),
				"class": "filter-select"
			});

			// Select the first project if there is one.
			if (projectsData.length > 0) {
				this._projectSelect.set("value", projectsData[0].id);
			}
			
			dojo.place(this._projectSelect.domNode, this._projectNode, "only");
			
			this._projectSelect.startup();
		}));			
	},
	
	_resetToRecent: function() {
		if (this._hasSearched) {
			// Reset search state, we want recent configurations again
			this._hasSearched = false;
			this.liveSearchBox.setSearchString("");
			dojo.removeClass(this.liveSearchBox.domNode, "search-filter-active");
			this._configurationsLabel.innerHTML = messages.recentConfigurationsText;
			// Set sorting
			this._configurationTable.initialSortBy = null; /* no sorting for recent configs */
			// TODO: Line above has no effect since ScrollableTable does not update header on column changed
			// TODO: Raise defect on foundation, for now set internal class directly
			this._configurationTable._tableHeader.sortBy = null;
			this.onColumnChanged();
		}
	},
	
	_projectChanged: function(project) {
		this._projectUri = project;
		
		this._resetToRecent();
		this._refresh();
	},
	
	_onScopeChanged: function(name, oldValue, newValue) {
		this._resetToRecent();
				
		this._currentScope = newValue;
		this.onColumnChanged();
		this._refresh();
	},
	
	_refresh: function(searchString, page /* 0 based index */) {
		var configurationType = this._currentScope.id;
	
		this._configurationTable.showLoading(true);

		if (this._hasSearched) {
			this.searchConfigurations(searchString || "", configurationType, page+1 || 1);
		} else {
			this.recentConfigurations(page || 0);
		}
	},
		
	_getIconClass: function() {
		var iconClass = "icon ";
		switch (this._currentScope.id) {
			case "stream":
				iconClass += "rmps-ui-resources-oslc-configuration-picker-stream-icon";
				break;
			case "baseline":
				iconClass += "rmps-ui-resources-oslc-configuration-picker-baseline-icon";
				break;
			default:
				iconClass += "rmps-ui-resources-oslc-configuration-picker-changeset-icon";
				break;
		}
		return iconClass;
	},
	
	/*
	 * function which takes in an 'item', and will return an appropriate <tr> element. 
	 * This 'item' represents a row, and will be the same as whatever 'items' are passed in
	 * through showRows(items). An additional requirement of the rowFactory is that for
	 * the <tr> elements it produces, all contained <td> elements
	 * (for columns that require sorting on) should contain an html attribute
	 * named "cell-value" that contains the value to be sorted on.
	 */
	rowFactory: function(item) {
		var tr = dojo.create("tr");
		
		// Title column
		var configNameTd = dojo.create("td",{"class": "config-row-cell dojoxEllipsis", "cell-value": item.title}, tr);
		var iconClass = this._getIconClass();
		var image = dojo.create("img", {"class": iconClass, "src": this._blankGif}, configNameTd);
		var span = dojo.create("span", {"class": "title-cell", innerHTML: item.title, title: item.title}, configNameTd);		

		// User/Parent column
		if (this._currentScope.id === "baseline") {
			dojo.create("td",{"class": "config-row-cell dojoxEllipsis", "cell-value": item.description, innerHTML: item.description, title: item.description}, tr);
		} else {
			var userid = item.creator.substr(item.creator.lastIndexOf('/') + 1);
			var rl = new ResourceLink({uri: item.creator, label: userid});
			var userTd = dojo.create("td",{"class": "config-row-cell dojoxEllipsis", "cell-value": userid}, tr);
			userTd.appendChild(rl.domNode);
		}
		dojo.create("td",{"class": "config-row-cell dojoxEllipsis", "cell-value": item.createdISO8601, innerHTML: DateUtils.formatDateLabel(item.created)}, tr);
		
		return tr;
	},
	
	/*
	 *  function which will return an array of objects describing the column headers. 
	 *  These objects should contain the parameters: label, sortable, filter, width
	 *  label - the label to display on this column header
	 *  sortable - whether this column header can be clicked to sort by
	 *  filterable - whether this column header should have a filter input field drop below it for filtering
	 *  width - width of this column, can mix percentages and widths
	 */
	columnFactory: function() {
		var configurationType = (this._currentScope && this._currentScope.id) || "stream";
		if (configurationType === "stream") {
			return [{label: messages.StreamName, sortable: true, filterable: true, width: "40%"},
			        {label: messages.CreatedBy, sortable: true, filterable: true, width: "25%"},
			        {label: messages.CreationDate, sortable: true, filterable: false, width: "35%"}
			        ];
		} else if (configurationType === "baseline") {
			return [{label: messages.BaselineName, sortable: true, filterable: true, width: "40%"},
			        {label: messages.FromStream, sortable: true, filterable: true, width: "25%"},
			        {label: messages.CreationDate, sortable: true, filterable: false, width: "35%"}
			        ];			
		} else if (configurationType === "changeSet") {
			return [{label: messages.ChangeSetName, sortable: true, filterable: true, width: "40%"},
			        {label: messages.CreatedBy, sortable: true, filterable: true, width: "25%"},
			        {label: messages.CreationDate, sortable: true, filterable: false, width: "35%"}
			        ];						
		}
	},
	
	searchConfigurations: function(searchString, type, page) {
		var projectId = this._projectUri.substr(this._projectUri.lastIndexOf('/') + 1);
		var def = OslcConfigClient.getConfigurations(projectId, searchString, type, page);
		def.addCallback(this, function(configurations) {
			this._paginationControl.startPage = page - 1;
			this._paginationControl.setTotal(configurations.totalResults);
			this._configurationTable.showRows(configurations);
		});
	},
	
	recentConfigurations: function(page) {
		var def = new dojo.Deferred();
		if (this._recentConfigurations === null) {
			// TODO: 348074: Add API to get recent local configurations used of a certain type in web UI
			def = ContextAPI.getRecentlyUsedLocalConfigurationsForProjectArea(this._projectUri, this.NUMBER_OF_RECENTS);
		} else {
			def = new dojo.Deferred();
			def.callback(this._recentConfigurations);
		}
		def.then(dojo.hitch(this, function(configurations) {
			this._recentConfigurations = configurations;
			if (configurations.length === 0) {
				this._paginationControl.startPage = 0;
				this._paginationControl.setTotal(0);
				this._configurationTable.showRows(configurations);
				return;
			}
			var configurationURIs = dojo.map(configurations, function(config) {
				return config.uri;
			});
			var def = OslcConfigClient.getRecentConfigurationsInfo(this._currentScope.id, configurationURIs, page);
			def.addCallback(this, function(configs) {
				this._paginationControl.startPage = page;
				this._paginationControl.setTotal(configs.totalResults);
				this._configurationTable.showRows(configs);
			});
		}));
	},
	
	doSearch: function(searchString, isPageChange) {
		// TODO: isPageChange?
		if (!this._hasSearched) {
			this._hasSearched = true;
			dojo.addClass(this.liveSearchBox.domNode, "search-filter-active");
			this._configurationsLabel.innerHTML = messages.matchingConfigurationsText;
			// Sort search results by date
			this._configurationTable.initialSortBy = 2; /* creation date column index */
			// TODO: Line above has no effect since ScrollableTable does not update header on column changed
			// TODO: Raise defect on foundation, for now set internal class directly
			this._configurationTable._tableHeader.sortBy = 2;
			this.onColumnChanged();
		}

		this._refresh(searchString, 0);
	},
	
	clearResults: function() {
		this._resetToRecent();
		this._refresh();
	},
		
	// ScrollableTable listens to this method
	onColumnChanged: function() {		
	},
	
	// Callback for when a row is double clicked on
	_onDoubleClickItem: function() {
		this.onClickOKButton();
	},
	
	// Callback for when a row is selected
	_onSelectionChanged: function(selectedItemOrUndefined) {
		this._selection = selectedItemOrUndefined;
		if (selectedItemOrUndefined) {
			this.okCmd.disabled = false;
		} else {
			this.okCmd.disabled = true;
		}
	},
	
	_close: function(items) {
		var response = {};
		response["oslc:message"] = "oslc.select";
		response["oslc:results"] = items;
		this._remoteFrame.sendResponse(dojo.toJson(response));		
	},
	
	onClickOKButton: function() {
		var items = [];
		if (this._selection) {
			var item = {
				"rdf:resource" : this._selection.uri,
				"rdf:type" : this._currentScope.id,
				"oslc:label" : this._selection.title
			};
			items.push(item);
		}
		this._close(items);
	},
	
	onClickCancelButton: function() {
		this._close([]);
	}
});
}());