/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014, 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function GlobalConfigurationResource() {
	dojo.provide("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.GlobalConfigurationResource");
	dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.VvcObject");
	var VvcObject = com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.VvcObject;
	
	dojo.require("dojox.html.entities");
	var EncodeHtmlEntity = dojox.html.entities.encode;
	
	dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.Constants");
	var Constants = com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.Constants;
	
	dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.GlobalConfigurationConstants");
	var GlobalConfigurationConstants = com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.GlobalConfigurationConstants;
		
	var DC_TERMS = "http://purl.org/dc/terms/";
	
	var TO_BE_ASSIGNED = "https://tobeassigned";

	dojo.declare("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.GlobalConfigurationResource", com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.VvcObject, {
				
		// Copy constructor
		constructor: function __GlobalConfigurationResource(args) {
			this.inherited(arguments);
			if (!args) {
				args = {};
			}
			
			this.setUri(args.uri ? args.uri : TO_BE_ASSIGNED);

			this.setProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type", GlobalConfigurationConstants.GC, "uri");
			
			//don't do the safeMixin -- do it the legible way since we are using setters anyway
			if (args.title) {
				this.setTitle(args.title);
			} else {
				this.setTitle("");	
			}
			
			if (args.description) {
				this.setDescription(args.description);
			} else {
				this.setDescription("");
			}
			
			if (args.mutable) {
				this.setMutable(args.mutable);
			} else {
				this.setMutable(false);	
			}
			
			if (args.component) {
				this.setComponent(args.component);
			}
			
			if (args.previousBaselines) {
				this.setPreviousBaselines(args.previousBaselines);
			} else {
				this.setPreviousBaselines([]);
			}
			
			if (args.contributions && dojo.isArray(args.contributions)) {
				var contributions = args.contributions;
				for(var i = 0; i < contributions.length; i++){
					var contribution = contributions[i];
					this.addContribution(contribution.configurationUri, contribution.order, contribution.hidden);
				}
			} else {
				this.setContributions([]);
			}
		},
		
		getFrontServiceId: function __getFrontServiceId() {
			return "globalConfigurations";
		},

		getTitle: function __getTitle() {
			return this.getProperty(Constants.TITLE);
		},
		
		setTitle: function __setTitle(newTitle) {
			this.setProperty(Constants.TITLE, EncodeHtmlEntity(newTitle), "literal");
		},		
		
		getDescription: function __getDescription() {
			return this.getProperty(Constants.DESCRIPTION);
		},
		
		setDescription: function __setDescription(newDesc) {
			this.setProperty(Constants.DESCRIPTION, newDesc, "literal");
		},
		
		getMutable: function __getMutable() {
			//be careful, it is stored as a string!
			return this.getProperty(GlobalConfigurationConstants.GC_MUTABLE) === "true";
		},
		
		setMutable: function __setMutable(newMutable) {
			this.setBooleanProperty(GlobalConfigurationConstants.GC_MUTABLE, newMutable);
		},		
		
		getComponent: function __getComponent() {
			return this.getProperty(GlobalConfigurationConstants.GC_COMPONENT);
		},
		
		setComponent: function __setComponent(newComponent) {
			this.setProperty(GlobalConfigurationConstants.GC_COMPONENT, newComponent, "uri");
		},		
		
		getPreviousBaselines: function __getPreviousBaselines() {
			return this.getArrayProperty(GlobalConfigurationConstants.GC_PREVIOUS_BASELINES);
		},
		
		setPreviousBaselines: function __setPreviousBaselines(value) {
			this.setPropertyArray(GlobalConfigurationConstants.GC_PREVIOUS_BASELINES, value, "uri");
		},
		
		/**
		 * Returns the unordered set of contributions as stored in RDF.
		 * 
		 * @see #getContributionConfigurationValues
		 */
		getContributions: function __getContributions() {
			var contributions = [];
			var contributionUris = this.getArrayProperty(GlobalConfigurationConstants.GC_CONTRIBUTION);
			if(contributionUris){
				for(var i = 0; i < contributionUris.length; i++){
					contributions.push(this.data[contributionUris[i].value]);
				}
			}
			return contributions;
		},
		
		/**
		 * Gets the unordered set of contribution configuration values
		 */
		getContributionConfigurationValues: function __getContributionConfigurationValues(){
			var contributions = this.getContributions();
			var contributionConfigurationValues = [];
			for(var i = 0; i < contributions.length; i++){
				contributionConfigurationValues.push(contributions[i][GlobalConfigurationConstants.GC_CONFIGURATION][0].value);
			}
			return contributionConfigurationValues;
		},
		
		getContributionValues: function __getContributionValues(){
			var contributions = this.getArrayProperty(GlobalConfigurationConstants.GC_CONTRIBUTION);
			var contributionValues = [];
			if(contributions && contributions.length > 0){
				for(var i = 0; i < contributions.length; i++){
					contributionValues.push(contributions[i].value);
				}
			}
			return contributionValues;
		},
		
		/**
		 * @return The unordered set of <code>ContributionResource</code>s
		 */
		getContributionResources: function __getContributionResources(){
			var resources = [];
			for(var key in this.data){
				if(key.indexOf("#") >= 0){
					var resource = new com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ContributionResource();
					resource.fromRDF(this.data[key]);
					resources.push(resource);
				}
			}			
			return resources;
		},
		
		/**
		 * <p>
		 * Be careful with this method, is only sets the contributions array and
		 * does not add OSLC-compliant contributions. To add or remove a
		 * contribution properly, see {@link #addContribution} and
		 * {@link #removeContribution}.  In most cases, you should probably use
		 * these methods to manipulate the contributions.
		 * </p>
		 */
		setContributions: function __setContributions(value) {
			this.setPropertyArray(GlobalConfigurationConstants.GC_CONTRIBUTION, value, "uri");
		},
		
		/**
		 * <p>
		 * Adds a contribution to the list of contributions. Maintains both the
		 * contribution list, and the objects which represent the contributions
		 * in the object. For more information, please see the spec available
		 * at:
		 * https://tools.oasis-open.org/version-control/browse/wsvn/oslc-ccm/trunk/specs/config-mgt.html
		 * </p>
		 * There are two steps to adding a contribution:
		 * <ol>
		 * <li>Add the contribution to the list of contributions.</li>
		 * <li>Define and add the contribution object.</li>
		 * </ol>
		 * 
		 * @param contributionUri
		 *            The URI of the resource being contributed.
		 * @param order
		 *            A <code>String</code> which will be used to
		 *            lexicographically sort the list of contributions. As of
		 *            2014, 2015-08-26, this field is restricted to ASCII characters
		 *            only.
		 * @param hidden
		 *            (optional) A <code>Boolean</code> which determines if
		 *            the newly added contribution is hidden or not. Defaults to
		 *            <code>False</code>.
		 */
		addContribution: function __addContribution(contributionUri, order, hidden){
			// Step 1:
			if(this.contributionExists(contributionUri)){
				if(djConfig.isDebug){
					console.warn("Duplicate contribution detected: '" + contributionUri + "'.  This contribution will not be added a second time.");
				}
				return;
			}
			
			var existingContributions = this.getContributionValues();
			existingContributions.push(contributionUri);
			this.setContributions(existingContributions);
			
			// Step 2:
			var contribution = new com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ContributionResource({
				uri: contributionUri,
				contributionUri: contributionUri,
				order: order,
				hidden: hidden
			});			

			/*
			 * Set the data. Clobber the existing contribution if needed. This
			 * provides inherent duplicate contribution safety.
			 */
			this.data = dojo.mixin(contribution.data, this.data);
		},
		
		/**
		 * <p>
		 * Removes and returns a particular contribution.
		 * </p>
		 * 
		 * @param contributionUri
		 *            The URI of the contribution to remove
		 * @return The removed contribution object, or <code>NULL</code> if
		 *         nothing was removed.
		 */
		removeContribution: function __removeContribution(contributionUri){
			if(this.contributionExists(contributionUri)){
				var newContributions = [];

				var contributions = this.getContributionValues()
				for(var i = 0; i < contributions.length; i++){
					var contributionConfigurationUri = this.data[contributions[i]][GlobalConfigurationConstants.GC_CONFIGURATION][0].value;
					if(contributionConfigurationUri === contributionUri){
						delete this.data[contributions[i]];
					} else {
						newContributions.push(contributions[i]);
					}
				}
				
				this.setContributions(newContributions);			
			}
			return null;
		},
		
		/**
		 * Determines if the contribution already exists in this GC.
		 */
		contributionExists: function __contributionExists(contributionUri){		
			var contributions = this.getContributionConfigurationValues();
			for(var i = 0; i < contributions.length; i++){
				var uri = contributions[i];
				if(uri === contributionUri){
					return true;
				}
			}
			return false;
		},
				
		//////////////// other properties
		
		getCreated: function __getCreated() {
			return this.getProperty(DC_TERMS + Constants.CREATED);
		},

		getCreator: function __getCreator() {
			return this.getProperty(DC_TERMS + Constants.CREATOR);
		},

//		getType: function __getType() {
//			return this.getProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
//		},
				
		doCreate: function __doCreate() {
			var wrapperDef = new dojo.Deferred();
			dojo.when(VvcObject.getVvcServerUri(), dojo.hitch(this, function(vvcUrl){
				var saveData = dojo.toJson(this.data);
				var theHeaders = {
					"Accept" : "application/rdf+json",
					"Content-Type" : "application/rdf+json",
				};
				var args = {
					url: vvcUrl + "/" + this.getFrontServiceId(),
					headers: theHeaders,
					handleAs: "json",
					postData: saveData,
					load: function __doCreateDefLoad(response, ioArgs) {
						if (ioArgs) {
							var xhr = ioArgs.xhr;
							if (xhr) {
								return xhr.getResponseHeader("Location");
							}
						}
					}
				};
				
				jazz.client.xhrPost(args).then(dojo.hitch(this, function __doCreateDefSuccess(data){
					wrapperDef.resolve(data);
				}), dojo.hitch(this, function __doCreateDefError(err){
					wrapperDef.reject(err);
				}));
			}));
			return wrapperDef;
		},
		
		onLoad: function __onLoad(response, ioArgs) {
			if(!this.uri || this.uri === TO_BE_ASSIGNED){
				throw new Error("URI has not been set!");
			}
			
			if (ioArgs && ioArgs.xhr) {
				this.eTag = ioArgs.xhr.getResponseHeader("ETag");
			}
			
			if (response) {
				this.data = response;
				this.properties = this.data[this.uri];
				this.loaded = true;
			}
			return this;
		},
		
		/*
		 * (amz) THIS IS A TERRIBLE HACK and ++ABSOLUTELY MUST++ be undone as
		 * soon as humanly possible!  This entails ripping out the VvcObject
		 * which is a very large task.
		 * 
		 * We need to guarantee that we know the URI when the GC loads otherwise
		 * if the GC has *ANY* contributions, this class will be initialized
		 * with the first set or properties it comes across.  With the new
		 * GC spec implemented, this means we could very easily initialise with
		 * the wrong set of properties.  If this happens, then the this object
		 * starts to fall apart and behave in very strange ways.
		 */
		_load: function __onLoadHack(uri, onError, synchronous) {
			var split = uri.split('?');
			if(split.length > 1){
				var queryString = split[1];
				var queryObj = dojo.queryToObject(queryString);
				if(queryObj.uri){
					this.setUri(queryObj.uri);
				} else {
					if(djConfig.isDebug){
						console.warn("GlobalConfigurationResource has been loaded but the GC's actual URI may not be known. Strange things could start to happen!" + uri)
					}
				}
			}

			return this.inherited(arguments);
		},
	});
	
	/**
	 * Represents a contribution resource.
	 */
	dojo.declare("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ContributionResource", com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.VvcObject, {
		
		constructor: function __ContributionResource(args){		
			if (!args) {
				args = {};
			}
			
			if(args.uri){
				this.setUri(args.uri);
			}
			
			if(args.contributionUri){
				this.setContributionUri(args.contributionUri);
			} else {
				this.setContributionUri(args.uri);
			}
			
			if(args.order){
				this.setContributionOrder(args.order);
			}
			
			if(args.hidden){
				this.setHiddenContribution(args.hidden);
			} else {
				this.setHiddenContribution(false);
			}
			
			this.setProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type", GlobalConfigurationConstants.GC_CONTRIBUTION, "uri");
		},
		
		fromRDF: function __fromRDF(rdf){
			this.data = rdf;
		},
		
		getContributionUri: function __getContributionUri(){
			this.getProperty(GlobalConfigurationConstants.GC_CONFIGURATION);
		},
		
		setContributionUri: function __setContributionUri(contributionUri){
			this.setProperty(GlobalConfigurationConstants.GC_CONFIGURATION, contributionUri, "uri");
		},
		
		getContributionOrder: function __getContributionOrder(){
			this.getProperty(GlobalConfigurationConstants.GC_CONTRIBUTION_ORDER);
		},
		
		setContributionOrder: function __setContributionOrder(order){
			this.setProperty(GlobalConfigurationConstants.GC_CONTRIBUTION_ORDER, order, "literal");
		},
		
		getIsHiddenContribution: function __getIsHiddenContribution(){
			return this.getBooleanProperty(GlobalConfigurationConstants.GC_HIDDEN);

		},
		
		setHiddenContribution: function __setHiddenContribution(isHidden){
			this.setBooleanProperty(GlobalConfigurationConstants.GC_HIDDEN, isHidden);
		},
		
		load: function __load(){
			if(djConfig.isDebug){
				console.warn("Loading a ContributionResource is unsupported.  Please load the contributions from the GlobalConfigurationResource instead!")
			}
		},
		
		loadAsync: function __load(){
			if(djConfig.isDebug){
				console.warn("Loading a ContributionResource is unsupported.  Please load the contributions from the GlobalConfigurationResource instead!")
			}
		},
		
		doSave: function __doSave(){
			if(djConfig.isDebug){
				console.warn("Saving a ContributionResource is unsupported.  Please save the contributions via the GlobalConfigurationResource instead!")
			}
		},
		
		doSaveAsync: function __doSave(){
			if(djConfig.isDebug){
				console.warn("Saving a ContributionResource is unsupported.  Please save the contributions via the GlobalConfigurationResource instead!")
			}
		},
		
		doPost: function __doSave(){
			if(djConfig.isDebug){
				console.warn("Saving a ContributionResource is unsupported.  Please save the contributions via the GlobalConfigurationResource instead!")
			}
		}
	});
})();