/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {	

	dojo.provide("com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ContextUtils");
	
	dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ConfigurationResource");
	dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.Setting");
	
	dojo.require("jazz.client.xhr");
	dojo.require("jazz.ui.toolbar.Button");

	dojo.require("net.jazz.ajax.contextRoot");
	dojo.requireLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.context", "ConfigurationContextMessages");
	
	dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils;

	dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.GlobalConfigurationConstants");
	var GlobalConfigurationConstants = com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.GlobalConfigurationConstants;
	
	var Setting = com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.Setting;
	
	var _spaceMapSetting = null;
	var messages = dojo.i18n.getLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.context", "ConfigurationContextMessages");
	
	com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ContextUtils = {
		
		MAX_RECENTLY_USED_SIZE : 5,
		spaceUri : null,
		_rootVvcUri: null,
		_currentContextCache : null,
		
		setSpace: function(space) {
			this.spaceUri = space;
		},
		
		getSpace: function() {
			return this.spaceUri;
		},
		
		getUserSetting: function() {
			if (!_spaceMapSetting) {
				var settingURI =  VvcUtils.getVvcUri() + "/settings/spaceMap";
				_spaceMapSetting = new Setting({
					uri: settingURI
				});
				_spaceMapSetting.load(settingURI);
			}
			return _spaceMapSetting;
		},
		
		getExpandedContext: function(resetCache) {
			var c = this._getCurrentContext(resetCache);
			return c ? c.expandedContext : null;
		},
		
		getCurrentContext: function(resetCache) {
			var c = this._getCurrentContext(resetCache);
			return c ? c.currentContext : null;
		},
		
		getExpandedOrCurrentContext: function() {
			var c = this.getExpandedContext();
			return c ? c : this.getCurrentContext();
		},
		
		_getCurrentContext: function(resetCache)
		{
			if (this._currentContextCache && !resetCache) {
				return this._currentContextCache;
		 	}
		 			
			if (!this.spaceUri) {
				return null;
			}
			
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}

			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (entry && entry.currentContextUri) {
				var config = {
					currentContext : this.getConfiguration(entry.currentContextUri),
					expandedContext : entry.expandedContextUri ? this.getConfiguration(entry.expandedContextUri) : null
				}
				if (config && config.currentContext)
				{
					this._currentContextCache = config;
					
					if (!config.expandedContext) {
						delete entry.expandedContextUri;
					}
					
					return config;
				}
				else
				{
					delete entry.expandedContextUri;
					delete entry.currentContextUri;
					if (setting) {
						setting.setData(spaceMap);
						setting.save();						
					}
				}
			}
			return null;
		},
		
		/*DEPRECATED, use getConfigurationAsync instead*/
		getConfiguration: function(configurationUri)
		{
			var config = new com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ConfigurationResource();
			var uri = VvcUtils.getVvcUri() + "/repository/configuration?uri="+configurationUri;
			config.load(uri, function(response, ioArgs) {
				console.info("Could not load '" + configurationUri + "' Response:" + response.message); 
			});
			
			if (config.loaded)
				return config;
			return null;
		},
		
		getConfigurationAsync: function(configurationUri){
			var config = new com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ConfigurationResource();
			var uri = VvcUtils.getVvcUri() + "/repository/configuration?uri="+configurationUri;
			return config.loadAsync(uri);
		},
		
		setCurrentContext: function(configuration, expandedConfiguration)
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (!entry) {
				entry = {};
				entry.spaceUri = this.spaceUri;
				spaceMap.push(entry);
			}
			
			var setCurrentContextCompleted = new dojo.Deferred();
			
			if (configuration && configuration.uri) {
				if (!expandedConfiguration) {
					if (entry.currentContextUri === configuration.uri && !entry.expandedContextUri) {
						setCurrentContextCompleted.resolve();
						return setCurrentContextCompleted; // current context is not changed
					}
				} else if (entry.currentContextUri === configuration.uri && expandedConfiguration.uri === entry.expandedContextUri) {
					setCurrentContextCompleted.resolve();
					return setCurrentContextCompleted; // current context is not changed
				}
				
				entry.currentContextUri = configuration.uri;
				entry.expandedContextUri = expandedConfiguration ? expandedConfiguration.uri : null;
				if (setting) {
					setting.setData(spaceMap);
					setting.save();				
				}
				
				this._currentContextCache = null;
				setCurrentContextCompleted.resolve();
				
			} else if (expandedConfiguration && expandedConfiguration.uri) {
				//No LC passed in, only GC so we need to lookup the appropriate LC for this space
				var getExistingContributionsArgs = {
						url: VvcUtils.getVvcUri() + "/contributions?" + dojo.objectToQuery({
							configurationUri: expandedConfiguration.uri
						}),
						headers: {
							"Accept" : "application/json"
						},
						handleAs: "json"
					};
				var getExistingContributionsDef = jazz.client.xhrGet(getExistingContributionsArgs);
				getExistingContributionsDef.then(dojo.hitch(this, function __getExistingConributionsSuccess(contributions) {
					if (contributions) {
						for (var i = 0; i < contributions.length; i++) {
							for (key in contributions[i]) {
								if (contributions[i][key].component[0].value === this.spaceUri) {
									//Found the first LC in the correct space
									foundLocalConfigForSpace = true;
									entry.currentContextUri = contributions[i][key].configurationUri[0].value;
									entry.expandedContextUri = expandedConfiguration.uri;
									
									if (setting) {
										setting.setData(spaceMap);
										setting.save();				
									}
									
									this._currentContextCache = null;
									setCurrentContextCompleted.resolve();
									return setCurrentContextCompleted;
								}
							}
						}
					}
					
					// Handle case where there is no local config for the space in the global config.						
					entry.expandedContextUri = expandedConfiguration.uri;
					
					if (setting) {
						setting.setData(spaceMap);
						setting.save();				
					}
					
					this._currentContextCache = null;
					setCurrentContextCompleted.resolve();
					return setCurrentContextCompleted;
				}), function __getExistingContributionsError(err) {
					//In the error case, make sure the deferred is resolved.
					setCurrentContextCompleted.reject(err);
					return setCurrentContextCompleted;
				});
			} else {
				setCurrentContextCompleted.resolve();
			}
			return setCurrentContextCompleted;
		},
		
		//warning: this is only here for compatibility.  you will only get the working configurations, not the extended configurations with this method
		//consider using getRecentlyUsedConfigurations
		getRecentlyUsed: function()
		{
			var recent = this.getRecentlyUsedConfigurations();
			var result = [];
			for (var i = 0; i < recent.length; i++) {
				result.push(recent[i].currentContext);
			}
			return result;
		},
		
		getRecentlyUsedConfigurations: function()
		{
			return this._loadRecentlyUsedUris(this._getRecentlyUsedUris());
		},
		
		_getRecentlyUsedUris: function()
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (entry && entry.recentlyUsedUris) {
				return entry.recentlyUsedUris;
			} else {
				return [];
			}
		},
		
		_loadRecentlyUsedUris: function(recentlyUsedUris)
		{
			var recentlyUsedConfigurations = [];
			for (i in recentlyUsedUris)
			{
				var recentlyUsedUri = recentlyUsedUris[i];
				if (recentlyUsedUri != null)
				{
					if (recentlyUsedUri.workingConfigurationUri) {
						var config = {
							currentContext: this.getConfiguration(recentlyUsedUri.workingConfigurationUri),
							expandedContext: recentlyUsedUri.extendedConfigurationUri ? this.getConfiguration(recentlyUsedUri.extendedConfigurationUri) : null
						};
						if (config.currentContext && !config.currentContext.isArchived() && ((config.expandedContext && !config.expandedContext.isArchived()) || !config.expandedContext)) {
							recentlyUsedConfigurations.push(config);
						} else {
							this._removeConfigurationFromRecentlyUsed(recentlyUsedUri);
						}
					}
				}
			}
			return recentlyUsedConfigurations;
		},
		
		_removeConfigurationFromRecentlyUsed: function(configUri)
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (!entry) {
				entry = {};
				entry.spaceUri = this.spaceUri;
				spaceMap.push(entry);
			}			
			
			if (!entry.recentlyUsedUris) {
				entry.recentlyUsedUris = [];
			}
			
			var i = entry.recentlyUsedUris.length - 1;
			while (i >= 0)
			{
				if (entry.recentlyUsedUris[i].workingConfigurationUri === configUri ||
						entry.recentlyUsedUris[i].extendedConfigurationUri === configUri) {
					entry.recentlyUsedUris.splice(i, 1);
				}
				i--;
			}
			if (setting) {
				setting.setData(spaceMap);
				setting.save();				
			}
		},
		
		addConfigurationToRecentlyUsed: function(configuration, expandedConfiguration)
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (!entry) {
				entry = {};
				entry.spaceUri = this.spaceUri;
				spaceMap.push(entry);
			}			
			
			if (!entry.recentlyUsedUris) {
				entry.recentlyUsedUris = [];
			}
			
			var c = {
				workingConfigurationUri: configuration.uri,
				extendedConfigurationUri: expandedConfiguration ? expandedConfiguration.uri : null
			}
			
			if (this.isConfigurationInRecentlyUsed(configuration, expandedConfiguration))
				return;

			
			entry.recentlyUsedUris.unshift(c);
			while (entry.recentlyUsedUris.length > this.MAX_RECENTLY_USED_SIZE)
			{
				entry.recentlyUsedUris.pop();
			}
			
			if (setting) {
				setting.setData(spaceMap);
				setting.save();				
			}
		},
		
		isConfigurationInRecentlyUsed: function(configuration, expandedConfiguration)
		{
			var recentlyUsedUris = this._getRecentlyUsedUris();
			if (recentlyUsedUris) {
				for (i in recentlyUsedUris)
				{
					if (configuration.uri === recentlyUsedUris[i].workingConfigurationUri) {
						if (expandedConfiguration && expandedConfiguration.uri === recentlyUsedUris[i].extendedConfigurationUri) {
							return true;
						} else if (!expandedConfiguration && !recentlyUsedUris[i].extendedConfigurationUri) {
							return true;
						}
						
					}
				}
			}
			return false;
		},
		
		getConfigurationIconClass: function(configuration) {
			if (configuration.isSnapshot())
				return "com-ibm-team-vvc-context-snapshot";
			else if (configuration.isChangeset())
			{
				if (configuration.isMutable())
					return "com-ibm-team-vvc-context-changeset";
				else
					return "com-ibm-team-vvc-context-completed-changeset";
			}
			return "com-ibm-team-vvc-context-workspace";
		},
		
		getExpandedConfigurationIconClass: function(configuration) {
			if (configuration.getProperty(GlobalConfigurationConstants.GC_MUTABLE) === "true")
				return "com-ibm-team-vvc-context-global-configuration-mutable";
			return "com-ibm-team-vvc-context-global-configuration-immutable";
		},		
		
		findSpaceEntry: function(spaceMap) {
			if (this.spaceUri != null && spaceMap != null) {
				for (var i in spaceMap) {
					var entry = spaceMap[i];
					if (entry && entry.spaceUri) {
						if (entry.spaceUri == this.spaceUri) {
							return entry;
						}
					}
				}
			}
			return null;
		},
		
		// the 2 functions below are copied from PageUtils.js
		getContextUri: function() {
			var hash = dojo.hash();
			var uri = null;
			if(hash){
				var hashObj = dojo.queryToObject(hash);
				uri = hashObj.contextUri ? hashObj.contextUri : hashObj.configurationUri;
			} 
			if (!uri) {
				var search = location.search;
				if(search){
					var searchObj = dojo.queryToObject(location.search.substring(1));
					uri =  searchObj.contextUri ? searchObj.contextUri : searchObj.configurationUri;
				}
			}
			return uri;
		},
		
		getRequestParmValue: function(paramName, uri) {
			var params = uri;
			if (!params) {
				params = window.location.href;
			}
			var i = params.indexOf("?");
			if (i !== -1) {
				params = params.substring(i+1);
				params = params.replace("#", "&");
			} else {
				i = params.indexOf("#");
				params = params.substring(i);
			}
			var nvps = [];
			var rawNvps = params.split("&");
			for(var i=0; i<rawNvps.length; i++) {
				var nvp = rawNvps[i].split("=");
				if (nvp[0] === paramName) {
					return nvp[1];
				}
			}
			return null;
		},
		
		/**
		 * Builds a button which, when clicked, will redirect the browser to the VVC history service for the supplied
		 * resource.
		 * @param args.conceptUri The concept URI For the resource
		 * @param args.contextUri (optional) The current context URI, defaults to the current context URI.
		 * @param args.displayName (optional) The name of the resource.
		 * @param args.displayNameHref (optional) The name of the resource wrapped in a link back to the resource.
		 * @return jazz.ui.toolbar.Button
		 */
		buildHistoryButtonForResource: function(args){
			if(!args.contextUri){
				args.contextUri = this.getCurrentContext().uri;
			}
							
			if(args.resourceDisplayName){
				if(!args.resourceDisplayNameHref){
					args.resourceDisplayNameHref = "";
				}	
			} 
					
			var uri = VvcUtils.getVvcUri() + "/web?" + dojo.objectToQuery(args) + "#action=exploreHistory";
			var button = new jazz.ui.toolbar.Button({
				imgSrc: dojo.moduleUrl("com.ibm.xtools.rmps.web.deliver.shared.web", "graphics/history.gif"),
				title: messages.showHistory,
				href: uri
			});
			return button;
		}	
	};


}());