/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014, 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function HistoryTable(){
   
dojo.provide("com.ibm.xtools.rmps.web.deliver.shared.web.ui.history.HistoryTable");

dojo.require("dijit._Widget");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require('dojo.i18n');

dojo.require('com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils');
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.UIUtils");

dojo.require('com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ContextUtils');

dojo.require ('com.ibm.xtools.rmps.web.deliver.shared.web.ui.VvcConstants');

dojo.require("jazz.ui.StyledTable");

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.UserUiNode");
var UserUiNode = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.UserUiNode;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.ConfigurationUiNode");
var ConfigurationUiNode = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.ConfigurationUiNode;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.ChangesetUiNode");
var ChangesetUiNode = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.ChangesetUiNode;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.DateTimeUiNode");
var DateTimeUiNode = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.DateTimeUiNode;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.UiDefinitionNode");
var UiDefinitionNode = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.UiDefinitionNode;

dojo.requireLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.history", "HistoryMessages");

(function HistoryTableAgain() {
	var messages = dojo.i18n.getLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.history", "HistoryMessages");
	var VvcConstants = com.ibm.xtools.rmps.web.deliver.shared.web.ui.VvcConstants;

	var ContextUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ContextUtils;
	var UIUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.UIUtils;
	
	dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils;
		
	dojo.declare("com.ibm.xtools.rmps.web.deliver.shared.web.ui.history.HistoryTable", [dijit._Widget], {
		
		constructor: function __historyTable(args)
		{
			if (args)
			{
				if (args.displayErrorMessage)
					this.displayErrorMessage = args.displayErrorMessage;
				if (args.hideErrorMessage)
					this.hideErrorMessage = args.hideErrorMessage;
			}
		},
		
		postCreate: function __postCreate(){			
			this._constructTable();
		},
	   		
		_constructTable: function __constructTable() {
			if (!this._table)
			{
			    this._table = new jazz.ui.StyledTable({
					   columns: [
					      {label: messages.name, sortable: false},
					      {label: messages.version, width: "100px", sortable: false},
					      {label: messages.dateCreated, width: "150px", sortable: false},
					      {label: messages.creator, width: "150px", sortable: false},
					      {label: messages.changeset, sortable: false}
					   ],
				});
				
				this._table.domNode.style.width = "100%";
				
				this._loadTable();
				
			    this.domNode.appendChild(this._table.domNode);
			    
			}
		},
		
		refresh: function __refresh()
		{
			UIUtils.showPageMessage("Refreshing explorer");
			this._loadTable();
			UIUtils.hidePageMessage();
		},
		
		_loadHistorySet: function __loadHistorySet()
		{
			var conceptUri = decodeURIComponent(ContextUtils.getRequestParmValue("conceptUri"));
			
			var configurationUri = ContextUtils.getContextUri();
			
			jazz.client.xhrGet({
				handleAs: 'json',
				sync: true,
				url: VvcUtils.getVvcUri() + "/changesets/history", //$NON-NLS-1$
				content: {
					"conceptUri" : conceptUri,
					"configurationUri" : configurationUri,
				},

				load: dojo.hitch(this, function(result) {
					this.data = result;
				}),
				
				error: dojo.hitch(this, function(e) {
					console.log(e);
					if (this.displayErrorMessage)
						this.displayErrorMessage("Error: "+e.message);
					else
						alert('error '+e.message);
				})
			});
			
			return this.data;  
		},
		
		_loadTable: function __loadTable()
		{
			var historySet = this._loadHistorySet();

			var conceptUri = ContextUtils.getRequestParmValue("conceptUri");
				
			var frag = document.createDocumentFragment();
			
			this._table.clearBody();
			
			// find default resource title and link
			var defaultTitle;
			var resourceDisplayName;
			var resourceDisplayNameHref;
			var params;
			if (window.location.hash && window.location.hash.length > 0) {
				params = window.location.hash.substring(1);
				resourceDisplayName = this._getParamValue(params, "resourceDisplayName");
				if (resourceDisplayName) {
					defaultTitle = jazz.util.html.escape(resourceDisplayName);
					resourceDisplayNameHref = this._getParamValue(params, "resourceDisplayNameHref");
				}
			}
			
			// find change set link parameters 
			var currentUri;
			var configName;
			if (historySet.length === 0) {
				if (defaultTitle) {
					dojo.publish("vvc.historyLastestVersionsTitle", [defaultTitle]);
				}
			} else if (params) {
				var i = window.location.href.indexOf("?");
				if (i !== -1) {
					currentUri = window.location.href.substring(0, i);
				}
				configName = this._getParamValue(params, "breadcrumb");
			}

			var tr, td;
			for (var i = historySet.length-1; i >= 0; i--) {
				
				var versionLine = historySet[i];

				var versionedTitle = VvcUtils.getDctermsDescriptionOnVersionedResource(versionLine.uri, conceptUri);
				
				// must escape html characters for title
				var resourceName = versionedTitle;
				if (!versionedTitle || versionedTitle === versionLine.uri) {
					if (defaultTitle && resourceDisplayName) {
						resourceName = resourceDisplayName;
						versionedTitle = defaultTitle;
					}
				} else {
					versionedTitle = jazz.util.html.escape(versionedTitle);
				}

				if (i== (historySet.length-1)) //publish the latest version as the concept title
					dojo.publish("vvc.historyLastestVersionsTitle", [versionedTitle]);
				
				tr = document.createElement("tr");
				dojo.addClass(tr, "versionRow");
				if (i%2 == 1)
					dojo.addClass(tr, "oddRow")
				
				td = document.createElement("td");
				dojo.addClass(td, "historyCell");
				var title;
				if (resourceDisplayNameHref === undefined) {
					title = this._wrapAnchorAroundText(versionLine.uri, versionedTitle);
				} else if (resourceDisplayNameHref) {
					title = this._wrapAnchorAroundText(resourceDisplayNameHref, versionedTitle);
				} else {
					title = versionedTitle;
				}
				td.innerHTML = title;
				tr.appendChild(td);
				
				// Version
				var index = versionLine.uri.lastIndexOf("/");
				td = dojo.create("td", {
					"class": "historyCell",
				}, tr);
				
				if(index == -1){
					// Deleted resource
					var deletedDefinitionNode = new UiDefinitionNode({
						titleText: messages.deletedTitle,
						definition: messages.deletedDefinition
					}, td);
				} else {
					dojo.setAttr(td, "innerHTML", jazz.util.html.escape(versionLine.uri.substring(index+1)));
				}
				
				// Creation time
				td = document.createElement("td");
				dojo.addClass(td, "historyCell");
				var datetimeNode = new DateTimeUiNode({
					timestamp: versionLine.creationTime
				}, td);
				tr.appendChild(td);
				
				// Creator
				td = document.createElement("td");
				var userUiNode = new UserUiNode({
					uri: versionLine.creatorUri
				}, td);
				tr.appendChild(td);
				
				// Changeset
				td = document.createElement("td");
				var changesetUiNode = new ChangesetUiNode({
					changeset: versionLine.changeset
				}, td);
				tr.appendChild(td);
				
				frag.appendChild(tr);
			}
			
			this._table.tbody.appendChild(frag);
		},
		
		/**
		 * <p>
		 * Extracts a named paramter from a set of URI parameters.
		 * </p>
		 * 
		 * @param params A list of params, can be <code>null</code>.
		 * @param paramName The name of the parameter to find.
		 * @return The properly unescaped, decoded value for the parameter name.
		 */
		_getParamValue: function __getParamValue(params, paramName) {
			if(params && paramName){
				var obj = dojo.queryToObject(params);
				if(obj && obj[paramName]){
					return obj[paramName];
				}
			}
			return null;
		},
		
		_getChangesetUrl: function __getChangesetUrl(changeset){
			if (changeset.state == "COMMITTED")
				return dojo.moduleUrl('com.ibm.xtools.rmps.web.deliver', 'shared/web/graphics/changeset_committed.gif');
			else if (changeset.state == "IN_PROGRESS")
				return dojo.moduleUrl('com.ibm.xtools.rmps.web.deliver', 'shared/web/graphics/changeset.gif');
			else if (changeset.state == "COMPLETED")
				return dojo.moduleUrl('com.ibm.xtools.rmps.web.deliver', 'shared/web/graphics/changeset_complete.gif');
			else if (changeset.state == "SUSPENDED")
				return dojo.moduleUrl('com.ibm.xtools.rmps.web.deliver', 'shared/web/graphics/changeset_suspended.gif');
			return "";
		},
		
		_wrapAnchorAroundText: function __wrapAnchorAroundText(uri, text)
		{
			return "<a href=\""+uri+"\">"+text+"<\a>";
		}
	});   
	})();
})();