/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2012, 2014, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function LinkToWorkItemUtil() {
dojo.provide('com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.LinkToWorkItemUtil');

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.oslc.web.ui.dialogs.AddCALMLinksDialog");
var AddCALMLinksDialog = com.ibm.xtools.rmps.web.deliver.shared.oslc.web.ui.dialogs.AddCALMLinksDialog;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.oslc.web.ui.actions.ShowLinkedElementsDialogAction");
var ShowLinkedElementsDialogAction = com.ibm.xtools.rmps.web.deliver.shared.oslc.web.ui.actions.ShowLinkedElementsDialogAction;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ContextUtils");
var ContextUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ContextUtils;

dojo.requireLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util", "LinkToWorkItemUtilMessages");
var Messages = dojo.i18n.getLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util", "LinkToWorkItemUtilMessages");

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.InfoContentUtils");
var InfoContentUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.InfoContentUtils;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ChangesetResource");
var ChangesetResource = com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ChangesetResource;

/**
 * This utility class is used to support links to work items.
 */
com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.LinkToWorkItemUtil = {
			
	//this method is intended for consumers, which typically are in a changeset configuration
	linkToWorkItemAction: function __linkToWorkItemAction(changesetResource, callback) {
		
		//otherwise, we could throw an error (although this should never really happen because the caller checks in isApplicable())
		if (changesetResource) {
			var subjectUri = changesetResource.id;
			var title = changesetResource.name;
			
			var params = {
				consumerMode: true,
	            dialogTitle: Messages.LinkToWorkItemDialogTitle,
	            //no need to add dialogSecondaryTitle or item
				onOk: function __onOk(items){
					items.forEach(com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.LinkToWorkItemUtil._linkToSingleWorkItem.bind(this, subjectUri, title, callback));
				},
	            resourceUri: subjectUri,
	            acceptableLinkTypes: ["http://open-services.net/ns/cm#"]
			};
			var dialog = new AddCALMLinksDialog(params);
			
		}
	},

	_linkToSingleWorkItem: function __linkToSingleWorkItem(subjectUri, title, callback, item) {
		var targetUrl = item.url;
		
		var args = {
			url: targetUrl + "?oslc.properties=oslc_cm:tracksChangeSet",
			headers: {
				"OSLC-Core-Version": "2.0",
				"Accept": "application/json",
				"Content-Type" : "application/json"
			}
		};
		
		var dfd = jazz.client.xhrGet(args);
		dfd.then(dojo.hitch(this, function __getWorkitemCallback(result) {
			var jsonObject = dojo.fromJson(result);
			var changeSets = jsonObject["oslc_cm:tracksChangeSet"];
			
			for (var i = 0; i < changeSets.length; i++) {
				if (changeSets[i]["rdf:resource"] === subjectUri) {
					//don't do anything since the work item is already linked to the change set
					return;
				}
			}
			
			changeSets.push({
				"dcterms:title" : title, 
				"rdf:resource" : subjectUri
			});

			var putObject = {
				"oslc_cm:tracksChangeSet" : changeSets
			};
			args["putData"] = dojo.toJson(putObject);
			args["headers"]["Content-Type"] = "application/json";
			var dfd2 = jazz.client.xhrPut(args);

			dfd2.then(dojo.hitch(this, function __putWorkitemCallback(result) {
				if(callback) {
					var link = {
								displayNameHref : item.url,
								displayName : item.label,
								changesetUri : subjectUri,
								subject : item.url
	                        };
					callback(link);
				}
			}), dojo.hitch(this, function(result) {
				InfoContentUtils.showSimpleContentDialog(Messages.errorAddingLinkTitle, Messages.errorAddingLink);
			}));
		}));
	},

	//this method is intended for vvc directly, so there is direct access to the changeset
	vvcLinkToWorkItemAction: function __vvcLinkToWorkItemAction(subjectUri, deferred) {
		
		var context = {
			deferred: deferred
		};
		
		var params = {
            dialogTitle: Messages.LinkToWorkItemDialogTitle,
            //no need to add dialogSecondaryTitle or item
            onOk: dojo.hitch(context, function __onOk(items){
            	var targetUrls = items.map(function __getItemUrl(item) {
            		return item.url;
            	});
            	if (targetUrls) {
					
					var urlArgs = {
						changeset: subjectUri,
						predicate: "http://jazz.net/ns/vvc/link/tracksChanges/rtc",
						backlinkPredicate: "http://open-services.net/ns/cm#tracksChangeSet",
						target: targetUrls,
						consumerUrl: VvcUtils.getVvcUri(),
						backlinkUrl: "http://open-services.net/ns/cm#tracksChangeSet"
					};
										
					var args = {
						headers: {
							"Content-Type": "application/x-oslc-common-link+xml",
							"Accept": "application/atom+json"
						},						
						url: VvcUtils.getVvcUri() + "/changesetLinks?" + dojo.objectToQuery(urlArgs),
						handleAs: "json",
						hasBody: false
					};
					var xhrPostDeferred = jazz.client.xhrPost(args);
					var context = {
						deferred: this.deferred
					};
					xhrPostDeferred.addCallback(context, dojo.hitch(this, function(result) {
						if (this.deferred) {
							this.deferred.resolve(result);
						}
					}));
					xhrPostDeferred.addErrback(context, dojo.hitch(this, function(error) {
						alert(Messages.errorAddingLink);
						if (this.deferred) {
							this.deferred.reject(error);
						}
					}));
					 
				}
			}),
            resourceUri: subjectUri,
            acceptableLinkTypes: ["http://open-services.net/ns/cm#"]
		};
		var dialog = new AddCALMLinksDialog(params); //
	}	
};

})();			