/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014, 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function VvcPagedCache(){
dojo.provide("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcPagedCache");

dojo.require("dojo.store.Memory");
dojo.require("dojo.store.Cache");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.notifications.notifications.VvcNotificationFactory");
var ncFactory = com.ibm.xtools.rmps.web.deliver.shared.web.ui.notifications.notifications.VvcNotificationFactory;

dojo.requireLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util", "VvcCacheMessages");
var Messages = dojo.i18n.getLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util", "VvcCacheMessages");

var _localStorageMap = [];

var PAGE_RANGE_START = "pageRangeStart";
var PAGE_RANGE_END = "pageRangeEnd";

dojo.declare("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcPagedCache", com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcCache, {
	
	_target: null,
	_idLock: null,
	_pageIdMap: null,
	
	constructor: function __VvcCache(args){
		this._pageIdMap = {};
	},

	uninitialize: function __uninitialize(){
		this.inherited(arguments);
		delete this._pageIdMap;
	},
	
	/**
	 * Generates the key for a particular page range.
	 * 
	 * @return a Generated key combining the resource ID and the page ranges
	 */
	getKeyForPage: function __getKeyForPage(id, pageRangeStart, pageRangeEnd){
		var params = {};
		params[PAGE_RANGE_START] = pageRangeStart;
		params[PAGE_RANGE_END] = pageRangeEnd;
		return id + "?" + dojo.objectToQuery(params);
	},
		
	/**
	 * Gets an object from the store based on its id. If the object does not
	 * exist, a query will goto the server to fetch it.
	 * 
	 * @return Always returns a deferred. When the cache has a result ( from the
	 *         local store or from the server) the deferred will resolve with
	 *         that result.
	 */
	get: function __get(id, query, pageRangeStart, pageRangeEnd){
		var resultsDeferred;
		
		var key = this.getKeyForPage(id, pageRangeStart, pageRangeEnd);
		
		var cachedResult = this._localStore.get(key);
		if(cachedResult){
			resultsDeferred = new dojo.Deferred();
			resultsDeferred.resolve(cachedResult);
		} else {
			/*
			 * Puts a lock on that ID and return the pending deferred. This
			 * ensures only one request per resource per transaction is pending
			 * at any one given time. Even if a bulk transaction is under way
			 * while a request for the new resource arrives, the result of the
			 * bulk transaction will be returned as the result for the new
			 * request. This prevents unnecessary requests to the server.
			 */
			if(this._idLock[key]){
				return this._idLock[key];
			}
			
			if(!query){
				query = {};
			}
			if(query[this._idProperty] === undefined){
				query[this._idProperty] = id;
			}
			
			if(!query[PAGE_RANGE_START]){
				query[PAGE_RANGE_START] = pageRangeStart;
			}
			
			if(!query[PAGE_RANGE_END]){
				query[PAGE_RANGE_END] = pageRangeEnd;
			}
			
			if (djConfig.isDebug) {
				query.debug = "true";
			}
			
			resultsDeferred =  this.query(id, query);
			this._idLock[key] = resultsDeferred;
			
			// We must always, error or success, unlock the ID when we're done with it!
			resultsDeferred.promise.always(dojo.hitch(this, function __unlockResource(){
				this._idLock[key] = null;
				delete this._idLock[key];
			}));
		}
		return resultsDeferred;
	},
	
	/**
	 * Executes a query against the remote data store.
	 * 
	 * Based on the store#query method.
	 * 
	 * @param The
	 *            query options.
	 * @return An array or promise.
	 * @see dojo.data.JsonRest
	 */
	query: function __query(id, query){
		var resultDef = new dojo.Deferred();
		var args = {
			url: this._target + "?" + dojo.objectToQuery(query),
			headers: {
				"Accept" : "application/json",
				"Content-Type" : "application/json"
			},
			handleAs: "json",
			preventCache: true
		};
		var xhrDeferred = jazz.client.xhrGet(args);
		xhrDeferred.then(dojo.hitch(this, function __processQueryResults(results){
			if(results){
				var key = this.getKeyForPage(id, query.pageRangeStart, query.pageRangeEnd);
				
				if(!this._pageIdMap[id]){
					this._pageIdMap[id] = [];
				}
				this._pageIdMap[id].push(key);
				
				if(results[id]){
					this.put(results[id], key);
					resultDef.resolve(results[id]);
				} else {
					// Allow for server-side macros
					this.put(results, key);
					resultDef.resolve(results);
				}
			}
		}), dojo.hitch(this, function __processQueryResultsError(error){
			resultDef.reject(error);
		}));
		
		return resultDef;
	},
	
	/**
	 * <p>Removes all the cached results for some id from the cache.  If the ID
	 * doesn't exist, or if the cache is empty for that ID, then the whole cache
	 * will be cleared.</p>
	 * 
	 * @param id
	 *            The ID of the item to remove.
	 * 
	 * @return The set of removed items.
	 */
	clear: function __clear(id){
		var toClear = this._pageIdMap[id];
		if(!toClear){
			this._localStore.clear();
			return null;
		}
		var removed = [];
		for(var i = 0; i < toClear.length; i++){
			var idToClear = toClear[i];
			this._localStore.clear(idToClear);
			removed.push(idToClear);
		}
		return removed;
	},
	
	/**
	 * The paged cache does not support bulk operations
	 */
	getAll: function __getAll(ids, queries){
		throw "VvcPagedCache does not support bulk operations!"
	},
	
	/**
	 * The paged cache does not support bulk operations
	 */
	startBulkTransaction: function __startBulkTransaction(){
		throw "VvcPagedCache does not support bulk operations!"
	},
	
	/**
	 * The paged cache does not support bulk operations
	 */
	endBulkTransaction: function __endBulkTransaction(error){
		throw "VvcPagedCache does not support bulk operations!"
	}
});
})();
