/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014, 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {	
dojo.provide("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.VvcConstants");
dojo.require("net.jazz.ajax.BootstrapProperties");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.interceptors.PreventCSRFInterceptor");

var VvcConstants = com.ibm.xtools.rmps.web.deliver.shared.web.ui.VvcConstants;

/**
 * <p>
 * Provides common utilities for VVC.
 * </p>
 */
com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils = {

	// Once the VVC Url is retrieved, we will return this.
	_cachedVvcUrl: sessionStorage.getItem(VvcConstants.VVC_URL_STORAGE_KEY),
	
	VVC_NAMESPACE : "http://jazz.net/ns/vvc#",   //$NON-NLS-1$
	RDF_NAMESPACE : "http://www.w3.org/1999/02/22-rdf-syntax-ns#",  //$NON-NLS-1$
	DCTERMS_NAMESPACE : "http://purl.org/dc/terms/",   //$NON-NLS-1$
	FOAF_NAMESPACE : "http://xmlns.com/foaf/0.1/",  //$NON-NLS-1$
	
	/**
	 * <p>
	 * Gets the full URI + context root of VVC.  If the URI is not known, then it will
	 * be fetched from the sever using the `queryVvc` service. Example:
	 * <code>"https://<server-name>:<server-port>/vvc-01"</code> will be returned, assuming VVC is 
	 * registered at "vvc-01". 
	 *  </p>
	 *  
	 *  @return The VVC URI, including context root.  
	 */
	getVvcUri: function(){
		return net.jazz.ajax.contextRoot();
	},
		
	/**
	 * Why is this method needed?  Shouldn't Jazz provide this for us?  Well...yes, but it turns out
	 * the method provided, `jazz.app.auth.logout()`, is mostly for RTC consumption.  There is another
	 * API that gets invoked when, from the jazz user menu, you select "logout" which isn't exposed 
	 * in any API, most applications don't need to set custom logout code so this is fine.  But for
	 * VVC, we have lots of delegated UI, for example the DeliverWizard, which does need to provide
	 * custom logout functionality.  As soon as Jazz releases a properly working logout method to be
	 * consumed by all, adopt it and remove this method!  Or, at the very least, make this method a
	 * thin, thin wrapper around the proper logout method.  
	 * 
	 * @return 
	 */
	logout: function(){
		var vvcUrl = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils.getVvcUri();
		var urlParams = {
			redirectURL: vvcUrl + "/web"
		}
		var logoutUrl = vvcUrl + "/service/com.ibm.team.repository.service.internal.ILogoutRestService?" + dojo.objectToQuery(urlParams);
		document.location = logoutUrl;
	},
	
	getDctermsDescriptionOnVersionedResource: function(resourceUri, conceptUri) {
		return this._getResourceLabel(resourceUri, "title", this.DCTERMS_NAMESPACE, conceptUri);
	},
	
	getDctermsDescription: function(resourceUri) {
		return this._getResourceLabel(resourceUri, "title", this.DCTERMS_NAMESPACE);
	},
	
	_getResourceLabel: function(resourceUri, tagName, tagNamespace, conceptUri) {
		this.label = resourceUri;
		if (conceptUri != null) {
			this.resource = conceptUri;
		} else {
			this.resource = resourceUri;
		}
		
		this.resource = decodeURIComponent(this.resource);
		
		try {
			var args = {
					headers: {
						"Accept": " application/rdf+xml"  //$NON-NLS-1$  $NON-NLS-2$
					},
					handleAs: 'text',
					sync: true,
					failOk: true,
					url: resourceUri
			};
			var deferred = jazz.client.xhrGet(args);
			deferred.then(dojo.hitch(this, function(result){
				var doc = dojox.xml.DomParser.parse(result);
				var entries = doc.getElementsByTagNameNS("RDF", this.RDF_NAMESPACE)[0].getElementsByTagNameNS("Description", this.RDF_NAMESPACE);  //$NON-NLS-1$  $NON-NLS-2$
				if (entries.length > 0){
					for (var i = 0; i<entries.length; i++) {
						var rdfAbout = entries[i].getAttribute("rdf:about"); //$NON-NLS-1$
						if (rdfAbout == this.resource) {
							var node = entries[i].getElementsByTagNameNS(tagName, tagNamespace);
							if (node.length > 0){
								var titleNode = node[0].childNodes[0];
								if (titleNode != null) {
									this.label = titleNode.nodeValue;
								}
								break;
							}
						}
					}
				}
			}), dojo.hitch(this, function(err, ioArgs){
				console.log("Failed to load resource '" + ioArgs.args.headers.vvc_servicePath + "' Invalid Response Status(" + ioArgs.xhr.status + "): "+ response.message); //$NON-NLS-1$  $NON-NLS-2$ $NON-NLS-3$
			}));
		} catch (err){
			//Stub, handled by "_onError()"
		}
		
		return this.label; 
	}
	
};
}());
