/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){   
dojo.provide("rmps.app.ui.internal.changesets.DmChangeSetsExplorerTable");

dojo.require('dojo.i18n');
dojo.require("dijit._Widget");

dojo.require("jazz.ui.toolbar.Separator");

dojo.require("dijit.MenuSeparator");
var Separator = dijit.MenuSeparator;

dojo.require("jazz.ui.ResourceLink");
var ResourceLink = jazz.ui.ResourceLink;

dojo.require("jazz.ui.Toolbar");
var Toolbar = jazz.ui.Toolbar;

dojo.require("jazz.ui.Menu");
var Menu = jazz.ui.Menu;

dojo.require("jazz.ui.toolbar.MenuButton");
var MenuButton = jazz.ui.toolbar.MenuButton;

dojo.require("com.ibm.team.repository.web.configurationManagement.CmConstants");
var ChangeSetStates = com.ibm.team.repository.web.configurationManagement.CmConstants.ChangeSetStateLiterals;

dojo.require("com.ibm.team.repository.web.configurationManagement.CmButtonFactory");
var CmButtonFactory = com.ibm.team.repository.web.configurationManagement.CmButtonFactory;
var CmMenuItemFactory = com.ibm.team.repository.web.configurationManagement.CmMenuItemFactory; // Comes from com.ibm.team.repository.web.configurationManagement.CmButtonFactory
dojo.require("com.ibm.team.repository.web.configurationManagement.notifications.CmNotificationFactory");
var CmNotificationFactory = com.ibm.team.repository.web.configurationManagement.notifications.CmNotificationFactory;

dojo.require("com.ibm.team.repository.web.configurationManagement.changesets.CmChangesetsExplorerTable");
dojo.require("com.ibm.team.repository.web.configurationManagement.grid.CmTreeGridTable");

dojo.require("com.ibm.team.repository.web.configurationManagement.uiNodes.UiNodeFactory");
var UiNodeFactory = com.ibm.team.repository.web.configurationManagement.uiNodes.UiNodeFactory;

dojo.require("rmps.app.ui.internal.util.CLMUtil");
var CLMUtil = rmps.app.ui.internal.util.CLMUtil;

dojo.requireLocalization("com.ibm.team.repository.web.configurationManagement.changesets", "ChangesetsPageMessages");
var CmChangeSetsPageMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.configurationManagement.changesets", "ChangesetsPageMessages");

dojo.require("rmps.app.ui.internal.util.ChangesetResourceUiNode");
var ChangesetResourceUiNode = rmps.app.ui.internal.util.ChangesetResourceUiNode;

dojo.require("rmps.app.ui.internal.changesets.DmChangesetLinkWidget");
var DmChangesetLinkWidget = rmps.app.ui.internal.changesets.DmChangesetLinkWidget;

dojo.require("rmps.app.ui.internal.util.DmButtonFactory");
var DmButtonFactory = rmps.app.ui.internal.util.DmButtonFactory;
	
dojo.declare("rmps.app.ui.internal.changesets.DmChangeSetsExplorerTable", [com.ibm.team.repository.web.configurationManagement.changesets.CmChangesetsExplorerTable], {
	
	
	registerRowBuilders: function(){
		var builders = this.inherited(arguments);
		builders.push(this._workItemLinksBuilder());
		return builders;
	},
	
	/**
	 * Builds the work item links area
	 */
	_workItemLinksBuilder: function(tr, data, table){
		var builder = {
			columnName: CmChangeSetsPageMessages.workItemLinksColumnTitle,
			columnClass: "workItemLinks",
			builderFunc: dojo.hitch(this, function(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function(td){
					if(tr._childOf){
						this._resourceLinksBuilder(tr, data, table, td);
					} else {
						this._changesetLinksBuilder(tr, data, table, td);
					}
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	},
	
	_changesetLinksBuilder: function(tr, data, table, td){
		var workItemWidget = new DmChangesetLinkWidget(data.id);
		td.appendChild(workItemWidget.domNode);
	},
	
	_resourceLinksBuilder: function(tr, data, table, td){
		// Stub, does nothing...
	},
	
	/**
	 * Builds the actions column for a row
	 */
	_actionsBuilder: function(tr, data, table){
		var builder = {
			columnName: CmChangeSetsPageMessages.actionsColumnTitle,
			columnClass: "configurationActionsColumn",
			builderFunc: dojo.hitch(this, function(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function(td){										
					var gc = tr._gc;
					
					// Outer toolbar, can be used for a selection box and the actions menu
					var toolbar = new jazz.ui.Toolbar({
						"class": "actionsToolbar",
						onfocus: dojo.hitch(this, function __onFocus(){
							tr.onFocus();
						}),
						onblur: dojo.hitch(this, function __onBlur(){
							tr.onBlur();
						})
					});
					
					if (tr._childOf) {
						if (!data.showMore && !data.showNoResults) {
							gc.destroys(toolbar);
							var showHistoryButton = CLMUtil.buildHistoryButtonForResource({
								conceptUri: data.uri,
								resourceDisplayName: data.displayName
							});
							toolbar.addChild(showHistoryButton);
							gc.destroys(showHistoryButton);
						}
					} else {
						this._gc.disconnects(dojo.connect(toolbar, "addChild", dojo.hitch(this, function(button){
							this._gc.destroys(button);
						})));
						
						gc.destroys(toolbar);
						
						// Toolbar which will be housed within the actions menu button
						var actionsMenu = new Menu();
						gc.destroys(actionsMenu);
						
						// Actions menu button					
						var actionsMenuButton = new MenuButton({
							menuProvider: function __getMenuProvider(){
								return actionsMenu;
							},
							cacheMenu: true,
							title: CmChangeSetsPageMessages.actionsColumnTitle,
							imgClass: "icons-show-actions",
							"class": "actionsToolbarMenuButton",
							onfocus: dojo.hitch(this, function(){
								tr.onFocus();
							}),
							onblur: dojo.hitch(this, function(){
								tr.onBlur();
							})
						});
						gc.destroys(actionsMenuButton);
						toolbar.addChild(actionsMenuButton);
	
						gc.disconnects(dojo.connect(actionsMenuButton, "open", this, function(){
							dojo.setAttr(tr.domNode, "menuOpen", null);
						}));
						
						gc.disconnects(dojo.connect(actionsMenuButton, "close", this, function(){
							dojo.removeAttr(tr.domNode, "menuOpen");
						}));
											
						/*
						 * TODO: Support (server & client side)for per-row refresh for
						 * change sets. For now, all calls to refresh the
						 * individual row will refresh the entire table instead.
						 */ 
	//					var refreshChangeSetbutton = CmMenuItemFactory.getRefreshButton(null, {
	//						actionCallback: dojo.hitch(this, function(){
	//							tr.onRefresh();
	//						})						
	//					});
	//					actionsMenu.addChild(refreshChangeSetbutton);
	//					
	//					// Separator
	//					actionsMenu.addChild(new Separator());
						
						/*
						 * Removed per: 
						 *  348554: Change Set Explorer widget should not create menus with a Complete Change Set action
						 */
	//					// Complete
	//					if(data.state === ChangeSetStates.open()){
	//						var completeChangesetButton = CmMenuItemFactory.getCompleteChangesetButton(data, {
	//							okCallback: function __onCompleteChangeSetCallback(){
	//								table.onHardRefresh();
	//							}		
	//						});
	//						actionsMenu.addChild(completeChangesetButton);
	//					}
						
					
						// Rename menu item
						var renameButton = CmMenuItemFactory.getRenameChangeSetButton(data, {
							okCallback: function __onRenameOkCallback(){
								table.onHardRefresh();
							}
						});
						actionsMenu.addChild(renameButton);
						
						var addLinkCallback = dojo.hitch(this, function(links) {
							table.onHardRefresh();
		                    if(this.linksTable !== null && this.linksTable._gridTable) {
			                    this.linksTable._gridTable.addRow(links);
			                    dojo.removeClass(this.domNode, "noLinks");
			                }
		                });
						
						var addLinkBtn = DmButtonFactory.getLinkToWorkItemButton(data, addLinkCallback, true);
						actionsMenu.addChild(addLinkBtn);
						
						// Unshared change sets
						if(data.state !== ChangeSetStates.committed()){
							// Deliver
							var deliverChangeSetButton = CmMenuItemFactory.getShareChangesetButton(data, {
								okCallback: function __onDeliverChangeSetOkCallback(){
									table.onHardRefresh();
								}			
							});
							actionsMenu.addChild(deliverChangeSetButton);
							
							// Discard
							var discardChangesetButton = CmMenuItemFactory.getDiscardChangesetButton(data, {
								okCallback: function __onDiscardChangeSetOkCallback(){
									table.onHardRefresh();
								},
								errorCallback: function __onDiscardChangeSetErrorCallback(error) {
									CmNotificationFactory.defaultXhrErrorHandler(error);
								}
							});
							actionsMenu.addChild(discardChangesetButton);
							
							// Separator
							actionsMenu.addChild(new Separator());
							
							// Compare with stream
							var compareChangesetWithStreamButton = CmMenuItemFactory.getCompareChangesetWithStreamButton(data);
							actionsMenu.addChild(compareChangesetWithStreamButton);
						}
						// add actions for committed changeset 
						else {
							var reverseChangesetButton = DmButtonFactory.getReverseChangesetButton(data, {
								actionCallback: dojo.hitch(this, function(){
									table.onHardRefresh();
								})						
							});
							actionsMenu.addChild(reverseChangesetButton);
						}
					}
								
					td.appendChild(toolbar.domNode);
					
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	},
	
	
	/**
	 * Builds the name column for the row
	 */
	_nameBuilder: function(tr, data, table){
		var builder = {
			columnName: CmChangeSetsPageMessages.nameColumnTitle,
			columnClass: "changesetNameColumn",
			builderFunc: dojo.hitch(this, function(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function(td){	
					if(tr._childOf){
						this._resourceNameBuilder(tr, data, table, td);
					} else {
						this._changesetNameBuilder(tr, data, table, td);
					}
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	},
		
	_changesetNameBuilder: function(tr, data, table, td){
		var gc = tr._gc;
		var expandoOrLoadingNode = dojo.create("span");
		gc.destroys(expandoOrLoadingNode);
		
		var expando = new com.ibm.team.repository.web.configurationManagement.grid.CmTreeGridTableExpando({
			explorerTable: this,
			tableRow: tr
		});
		gc.destroys(expando);
		expandoOrLoadingNode.appendChild(expando.domNode);
		
		var busyIndicatorNode = new com.ibm.team.repository.web.configurationManagement.grid.CmTreeGridTableBusyIndicator({
			explorerTable: this,
			tableRow: tr
		});
		gc.destroys(busyIndicatorNode);
		expandoOrLoadingNode.appendChild(busyIndicatorNode.domNode);
		
		td.appendChild(expandoOrLoadingNode);
							
		gc.disconnects(dojo.connect(tr, "_onShowBusyIndicator", function _onShowBusyIndicator(){
			dojo.style(busyIndicatorNode.domNode, "display", "");
		}));
		
		dojo.style(busyIndicatorNode.domNode, "display", "none"); // Defualt hide the busy indicator
		gc.disconnects(dojo.connect(tr, "_onHideBusyIndicator", function _onHideBusyIndicator(){
			dojo.style(busyIndicatorNode.domNode, "display", "none");
		}));
		
		var configuratonTitleNode = UiNodeFactory.getChangeSetUiNode({
			uri: data.id
		}, td);
		gc.destroys(configuratonTitleNode);
		
		gc.disconnects(dojo.connect(configuratonTitleNode.domNode, 'onfocus', dojo.hitch(this, function(){
			tr.onFocus();
		})));
		gc.disconnects(dojo.connect(configuratonTitleNode.domNode, 'onblur', dojo.hitch(this, function(){
			tr.onBlur();
		})));
		
		gc.disconnects(dojo.connect(configuratonTitleNode, 'onfocus', dojo.hitch(this, function(){
			tr.onFocus();
		})));
		gc.disconnects(dojo.connect(configuratonTitleNode, 'onblur', dojo.hitch(this, function(){
			tr.onBlur();
		})));			
	},
	
	_resourceNameBuilder: function(tr, data, table, td){
		var childIndenter = dojo.create("span", {
			// Directly setting the style here is a secessary evil since we need to generate the width.
			style: "width:40px; height:26px; display:inline-block;",
			innerHTML: "&nbsp;"
		});
		td.appendChild(childIndenter);
		
		if(data.showNoResults){
			var noResultsNode = dojo.create("span", {
				innerHTML: CmChangeSetsPageMessages.noResourcesFound,
				"class": "noResultsNode"
			}, td);
		} else if (data.showMore) {
			var showMore = new ResourceLink({
				uri: data.changesetUri,
				label: data.displayName
			});
			td.appendChild(showMore.domNode);
		} else {
			var resourceNode = new ChangesetResourceUiNode({
				"data": data
			});
			td.appendChild(resourceNode.domNode);
		}
	},
	
	_creatorBuilder: function(tr, data, table){
		var builder = {
			columnName: CmChangeSetsPageMessages.creatorColumnTitle,
			columnClass: "creatorColumn",
			builderFunc: dojo.hitch(this, function(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function(td){	
					if (!tr._childOf) {
						var userUiNode = UiNodeFactory.getUserUiNode({
							uri: data.creator
						});
						td.appendChild(userUiNode.domNode);
					}
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	}
});
}());
