/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() { 
dojo.provide("rmps.app.ui.internal.linkvalidity.LinkValidityBulkStatusWidget");

dojo.require("rmps.app.ui.internal.common.MessageForm");
dojo.require("rmps.app.ui.internal.linkvalidity.LinkValidityUtils");
dojo.require("rmps.app.ui.internal.linkvalidity.LinkValidityStatusComboButton");
dojo.require("rmps.app.ui.internal.links.LinkTypeUtils");
dojo.require("rmps.app.ui.internal.util.vvcutils");
dojo.require("rmps.app.ui.internal.common.MessageHandler");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.linkvalidity", "LinkValidityControlWidget");

var LinkValidityUtils = rmps.app.ui.internal.linkvalidity.LinkValidityUtils,
	LinkValidityStatusComboButton = rmps.app.ui.internal.linkvalidity.LinkValidityStatusComboButton,
	LinkTypeUtils = rmps.app.ui.internal.links.LinkTypeUtils,
	VvcUtils =rmps.app.ui.internal.util.vvcutils,
	MessageHandler = rmps.app.ui.internal.common.MessageHandler;

dojo.declare("rmps.app.ui.internal.linkvalidity.LinkValidityBulkStatusWidget", [rmps.app.ui.internal.common.MessageForm], {
	
	status: LinkValidityUtils.LINK_STATUS.DO_NOT_CHANGE,
	dialogWidth: "350px",
	dialogStyle: "ok",
	
	projectUri: null,
	dataCache: null,		// The ArtifactDataCache
	resources: null,		// Array of the artifacts whose links will be updated
	linkTypeCols: null,		// Array of the link type columns to update
	
	_commentInput: null,	// The domNode for the comment text area
	_typeStatusMap: null,	// A map of the status widgets for each link type column
	
	postMixInProperties: function() {
		this.inherited("postMixInProperties", arguments);
		this.messages = dojo.i18n.getLocalization("rmps.app.ui.internal.linkvalidity", "LinkValidityControlWidget");
		this.dialogTitle = this.messages.bulkValidityStatusWidgetDialogTitle;
		this.confirmContent = dojo.create("div");
	},
	
	postCreate: function() {
		this.inherited("postCreate", arguments);
		
		this.resources = this.resources || [];
		this.linkTypeCols = this.linkTypeCols || [];
		
		// Get the link types then use them to get info about our columns.
		this._typeStatusMap = {};
		var typesTable = dojo.create("table");
		var linkTypesDeferred = LinkTypeUtils.getLinkTypesForProject(this.projectUri);
		linkTypesDeferred.addCallback(this, function(types) {
						
			// Determine the proper link URIs to use and find their proper labels using the directionality of the column info.
			dojo.forEach(this.linkTypeCols, dojo.hitch(this, function(col) {
				
				var linkTypeKey = col.key,
					typeUri = col.linkTypeURI,
					typeName;
				var directionIndex = linkTypeKey.lastIndexOf("_");
				var isObject = (directionIndex !== -1 && (linkTypeKey.indexOf("_OBJ") === directionIndex)),
					isBidirectional = linkTypeKey.indexOf("_BI") === directionIndex;
				
				// Associate the type object relating to the link we're dealing with
				var linkType = dojo.filter(types, function (type) {
					if (type.getResourceURI() === col.linkTypeURI) {
						return true;
					} else if (col.isExternalLinkType) {
						var sameAs = type.getSameAs();
						if (sameAs && sameAs === col.linkTypeURI) {
							return type.getSameAs() ? sameAs === col.linkTypeURI : false;
						}
					}
				});
				linkType = linkType[0];
				if (!linkType) {
					return;
				}
				typeUri = linkType.getSameAs() ? linkType.getSameAs() : linkType.getResourceURI();
				
				// Filter our OSLC links when we are in a local config
				if (!VvcUtils.getGlobalConfigurationUri()) {
					if (LinkTypeUtils.isCalmLink(typeUri)
							|| LinkTypeUtils.isDMLinkType(typeUri)
							|| LinkTypeUtils.isRMtoRMLinkType(typeUri)) {
						return;
					}
				}
				
				if (col.isExternalLinkType) {
					typeName = linkType.getName();
				} else {
					typeName = isObject ? linkType.getObjectToSubjectLabel() : linkType.getSubjectToObjectLabel();
				}
				
				// Add a row for each of the link types, so they can be controlled independently
				var statusWidget = new LinkValidityStatusComboButton({status: this.status, showDoNotChangeOption: true});
				this._typeStatusMap[col.key] = {
						statusWidget: statusWidget,
						linkType: typeUri,
						isObject: isObject,
						isBidirectional: isBidirectional
				};
				typesTable.appendChild(this._createLinkTypeRow(typeName, statusWidget.domNode));
			}));
			
			if (!Object.keys(this._typeStatusMap).length) {
				// No valid link type columns are being displayed.
				this._setNoLinkColumnsContent();
			} else {
				// Add the text showing how many artifacts are affected
				this.confirmContent.appendChild(document.createTextNode(dojo.string.substitute(this.messages.settingStatusForNArtifacts, [this.resources.length])));
				
				// Add the table that we constructed with the type labels and combo boxes
				this.confirmContent.appendChild(typesTable);
				
				// Add the comment input
				this._commentInput = dojo.create("textarea", {rows: 4, placeholder: this.messages.enterCommentPlaceholder, style: "width: 100%; max-width: 100%; margin-top: 5px;"}, this.confirmContent);
			}
		});
	},
	
	_onOk: function() {
		this.inherited("_onOk", arguments);
		
		if (!Object.keys(this._typeStatusMap).length || !this.resources.length) {
			// Nothing to update.
			return;
		}
		
		// Using the data cache, collect the links for each column for our resources,
		//	then use that to construct our post data to send to updateValidity
		var postData = [],
			keys = Object.keys(this._typeStatusMap),
			comment = this._commentInput.value || "";
		
		var resources = [];
		for (var i = 0; i < this.resources.length; i++) {
			resources.push(this.resources[i].getResourceURI());
		}
		
		for (var i = 0; i < keys.length; i++) {
			var key = keys[i];
			var mapEntry = this._typeStatusMap[key];
			var status = mapEntry.statusWidget.getStatus();
			
			// Filter out any entries with the "DO_NOT_CHANGE" status
			if (status === LinkValidityUtils.LINK_STATUS.DO_NOT_CHANGE) {
				continue;
			}
						
			// See if there is already an entry for this link type in the post data
			//	if so, update it. Otherwise, create a new entry;
			var entry = null;
			for (var j = 0; j < postData.length; j++) {
				if (mapEntry.linkType === postData[j].linkType) {
					entry = postData[j];
					entry.status = status;
					break;
				}
			}
			
			if (!entry) {
				entry = {
					status: status,
					validityDescription: comment,
					linkType: mapEntry.linkType
				};
				postData.push(entry);
			}
						
			if (mapEntry.isObject) {
				entry.targetConcepts = resources;
			} else {
				entry.srcConcepts = resources;
			}
		};
		
		if (postData.length) {
			LinkValidityUtils.bulkUpdateValidity(postData).then(function(res) {
				// Check if any of the links could not be set. If so, display a page-level warning.
				if (res.length) {
					for (var i = 0; i < res.length; i++) {
						if (res[i].error) {
							MessageHandler.showWarning(7924);
						}
					}
				}
			});
		}
	},
	
	show: function() {
		this.inherited("show", arguments);
		
		// Place focus on the first status combo box
		var keys = Object.keys(this._typeStatusMap);
		if (keys.length) {
			this._typeStatusMap[keys[0]].statusWidget.focus();
		}
	},
	
	_createLinkTypeRow: function(label, node) {
		var row = dojo.create("tr");
		dojo.create("td", {"class": "com-ibm-rdm-web-linkvalidity-type-label"}, row).appendChild(document.createTextNode(label + ":"));
		dojo.create("td", null, row).appendChild(node);
		return row;
	},
	
	_setNoLinkColumnsContent: function() {
		this.confirmContent.appendChild(document.createTextNode(this.messages.linkValidityNoLinkColSelected));
	}
	
});
})();
