/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() { 
dojo.provide("rmps.app.ui.internal.linkvalidity.LinkValidityRollupWidget");

dojo.require("dojo.i18n");

dojo.require("rmps.app.ui.internal.linkvalidity.LinkValidityUtils");
dojo.require("rmps.app.ui.internal.linkvalidity.LinkValidityProfile");
dojo.require("rmps.app.ui.internal.notifications.ChangeNotifier");
dojo.require("rmps.app.ui.internal.notifications.EventTopics");
dojo.require("rmps.app.ui.internal.common.HoverPopup");
dojo.require("rmps.app.ui.internal.server.client.utils.LinkUtils");
dojo.require("rmps.app.ui.internal.links.LinkTypeUtils");
dojo.require("rmps.app.ui.internal.links.linkutils");
dojo.require("rmps.app.ui.internal.model.resources.ResourceManager");
dojo.require("rmps.app.ui.internal.util.vvcutils");
dojo.require("rmps.app.ui.internal.common.ArtifactIcons");
dojo.require("rmps.app.ui.internal.common.ResourceLinkNoHover");


dojo.requireLocalization("rmps.app.ui.internal.linkvalidity", "LinkValidityControlWidget");

var LinkValidityUtils = rmps.app.ui.internal.linkvalidity.LinkValidityUtils,
    LinkValidityProfile = rmps.app.ui.internal.linkvalidity.LinkValidityProfile,
	ChangeNotifier = rmps.app.ui.internal.notifications.ChangeNotifier,
	EventTopics = rmps.app.ui.internal.notifications.EventTopics,
	HoverPopup = rmps.app.ui.internal.common.HoverPopup,
	ServerLinkUtils = rmps.app.ui.internal.server.client.utils.LinkUtils,
	LinkTypeUtils = rmps.app.ui.internal.links.LinkTypeUtils,
	LinkUtils = rmps.app.ui.internal.links.linkutils,
	ResourceManager = rmps.app.ui.internal.model.resources.ResourceManager,
	ResourceLinksHandler = rmps.app.ui.internal.common.ResourceLinksHandler,
	VvcUtils = rmps.app.ui.internal.util.vvcutils,
	ArtifactIcons = rmps.app.ui.internal.common.ArtifactIcons,
	ResourceLinkNoHover = rmps.app.ui.internal.common.ResourceLinkNoHover;

dojo.declare("rmps.app.ui.internal.linkvalidity.LinkValidityRollupWidget", [dijit._Widget], {
	
	projectUri: null,
	concept: null,
	entryMap: null,
	_messages: null,
	_listeners: null,
	_hover: null,
	_statusMessage: null,
	
	
	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("rmps.app.ui.internal.linkvalidity", "LinkValidityControlWidget");
	},
	
	postCreate: function() {
		this.inherited("postCreate", arguments);
		
		this._listeners = [];
		this.domNode = dojo.create("div");
		var entry = this._getEntry();
		if (!entry) {
			// Note that artifacts with no contributing links should now show as valid. To
			//	accommodate that, we need to give it a stub entry in the entry map
			entry = {conceptUri: this.concept};
			this._setEntry(entry);
		}
		
		this.update(entry);
		
		// Init the hover
		this._initializeHover();
		
		this._listeners.push(ChangeNotifier.addListener({
			eventTopic: EventTopics.LINK_VALIDITY_UPDATED,
			callbackFunction: dojo.hitch(this, this._updateLinkValidity),
			callbackScope: this,
			projectURI: this.projectURI
		}));
		
		this._listeners.push(ChangeNotifier.addListener({
			eventTopic: EventTopics.LINK_CREATED,
			callbackFunction: dojo.hitch(this, this._updateLinkCreated),
			callbackScope: this,
			projectURI: this.projectURI
		}));
		
		this._listeners.push(ChangeNotifier.addListener({
			eventTopic: EventTopics.LINK_DELETED,
			callbackFunction: dojo.hitch(this, this._updateLinkDeleted),
			callbackScope: this,
			projectURI: this.projectURI
		}));
		
		this._listeners.push(ChangeNotifier.addListener({
			eventTopic: EventTopics.ARTIFACT_SAVED,
			callbackFunction: dojo.hitch(this, this._updateArtifactSaved),
			callbackScope: this,
			projectURI: this.projectURI
		}));
		
		this._listeners.push(ChangeNotifier.addListener({
			eventTopic: EventTopics.LINK_VALIDITY_ROLLUP_UPDATED,
			callbackFunction: dojo.hitch(this, this._updateEntry),
			callbackScope: this,
			projectURI: this.projectURI
		}));
	},
	
	uninitialize: function() {
		if (this._hover) {
			this._hover.destroy();
		}
		dojo.forEach(this._listeners, dojo.hitch(this, function(listener) {
			ChangeNotifier.removeListener(listener);
		}));
	},
	
	_getEntry: function() {
		return this.entryMap[this.concept];
	},
	
	_setEntry: function(entry) {
		this.entryMap[this.concept] = entry;
	},
	
	_updateEntry: function(args) {
		var entry = args.notifierParams.entry;
		if (!entry) {
			return;
		}
		
		// If the entry represents this resource, update this widget.
		if (entry.conceptUri === this.concept) {
			this._setEntry(entry);
			this.update(this._getEntry());
			return;
		}
		
		if (!entry.validityDetails) {
			return;
		}
		
		// Now, see if we the entry that was updated contained links to this resource. If so, update.
		for (var i = 0; i < entry.validityDetails.length; i++) {
			var details = entry.validityDetails[i];
			if (this.concept === details.srcConcept || this.concept === details.targetConcept) {
				LinkValidityUtils.getValidityRollupData([{conceptUri: this.concept}], true)
					.then(dojo.hitch(this, function(results) {
						if (results.length === 1) {
							this._setEntry(results[0]);
							this.update(this._getEntry());
						}
					}));
				break;
			}
		}
	},
	
	_updateLinkValidity: function(args) {
		var entries = args.notifierParams.entries;
		if (!entries || !entries.length) {
			return;
		}
		
		for (var i = 0; i < entries.length; i++) {
			var entry = entries[i];
			if (this.concept === entry.srcConcept || this.concept === entry.targetConcept) {
				// The link that has been updated belongs to the resource this widget represents.
				//	Request the new rollup info from the server and update the node
				LinkValidityUtils.getValidityRollupData([{conceptUri: this.concept}], true)
					.then(dojo.hitch(this, function(results) {
						if (results.length === 1) {
							this._setEntry(results[0]);
							this.update(this._getEntry());
						}
					}));
				break;
			}
		}
	},
	
	_updateLinkCreated: function(args) {
		var links = args.notifierParams.links;
		if (!links || !links.length) {
			return;
		}
			
		for (var i = 0; i < links.length; i++) {
			var link = links[i];
			if (this.concept === link.getObject() || this.concept === link.getSubject()) {
				// A link has been created on the resource that this widget represents. Update the entry in the map, then notify listeners.
				LinkValidityUtils.getValidityRollupData([{conceptUri: this.concept}], true)
					.then(dojo.hitch(this, function(results) {
						if (results.length === 1) {
							ChangeNotifier.notifyListeners({
								eventTopic: EventTopics.LINK_VALIDITY_ROLLUP_UPDATED,
								entry: results[0]
							});
						}
					}));
				break;
			}
		}
	},
	
	_updateLinkDeleted: function(args) {
		var link = args.notifierParams.link;
		if (!link) {
			return;
		}
			
		if (this.concept === link.getObject() || this.concept === link.getSubject()) {
			// A link has been deleted on the resource that this widget represents. Update the entry in the map.			
			LinkValidityUtils.getValidityRollupData([{conceptUri: this.concept}], true)
				.then(dojo.hitch(this, function(results) {
					if (results.length === 1) {
						ChangeNotifier.notifyListeners({
							eventTopic: EventTopics.LINK_VALIDITY_ROLLUP_UPDATED,
							entry: results[0]
						});
					}
				}));
		}
	},
	
	_updateArtifactSaved: function(args) {
		var savedUri = args.notifierParams.resourceURI;
		if (this.concept === savedUri) {
			// This entry needs to be updated. Fetch the new roll-up data.
			//	Get the detailed data, because other roll-ups may need to be updated as well.
			LinkValidityUtils.getValidityRollupData([{conceptUri: this.concept}], true)
				.then(dojo.hitch(this, function(results) {
					if (results.length === 1) {
						// Notify all widgets, including this one, so they can update if necessary.
						ChangeNotifier.notifyListeners({
							eventTopic: EventTopics.LINK_VALIDITY_ROLLUP_UPDATED,
							entry: results[0]
						});
					}
				}));
		}
	},
	
	update: function(entry) {
		var iconClass,
			text;
		this._initializeHover();
		if (entry.totalInvalid) {
			iconClass = "com-ibm-rdm-web-linkvalidity-invalid-summary-icon";
			this._statusMessage = this._messages.invalidRollupMessage;
		} else if (entry.totalUnknown) {
			iconClass = "com-ibm-rdm-web-linkvalidity-unknown-summary-icon";
			this._statusMessage = this._messages.unknownRollupMessage;
		}
        else if (entry.totalValid) {
			iconClass = "com-ibm-rdm-web-linkvalidity-valid-summary-icon";
			this._statusMessage = this._messages.validRollupMessage;
		} else {
			// There are no links for this resource. Show nothing.
			iconClass = "com-ibm-rdm-web-linkvalidity-valid-summary-icon";
			this._statusMessage = this._messages.noLinks;
		}
		
		if (!this.domNode) {
			return;
		}
		dojo.removeClass(this.domNode);
		dojo.addClass(this.domNode, iconClass);
	},
	
	_initializeHover: function() {
		if (this._hover) {
			this._hover.destroy();
		}
		this._hover = new HoverPopup({
			around: this.domNode,
			createContents: this._createPopupContent.bind(this),
			destroyContents: true
		});
	},
	
	_createPopupContent: function() {
		// Creates the content of the popup hover based on the rollup status entry
		//	and a generated map of the links for the resource this rollup represents
		var container = dojo.create("div", {"class": "com-ibm-rdm-web-linkvalidity-hover"});
		var closeButton = dojo.create("div", {"class": "close-button"}, container);
		this.connect(closeButton, "onclick", dojo.hitch(this, function(evt) {
			this._hover.close();
		}));
		
		// Add the status message and info icon
		var statusMessageTable = dojo.create("table", {"class": "status-message-table"}, container);
		var statusMessageRow = dojo.create("tr", {"class": "status-message-row"}, statusMessageTable);
		dojo.create("td", {"class": "status-message"}, statusMessageRow).appendChild(document.createTextNode(this._statusMessage));
		dojo.create("td", null, statusMessageRow).appendChild(dojo.create("div", {"class": "info-icon", "title": this._messages.rollupHoverTooltip}));
		
		
		// Get the URI for the link validity help topic, then create a link to it in the status row.
		LinkValidityUtils.getHelpTopicUri().then(dojo.hitch(this, function(res) {
			if (res) {
				var anchor = dojo.create("a", {"class": "more-details-link", "title": this._messages.moreDetails, "href": res});
				anchor.appendChild(document.createTextNode(this._messages.moreDetails));
				dojo.create("td", null, statusMessageRow).appendChild(anchor);
			}
		}));
		
		var linksDiv = dojo.create("div", null, container);
		
		// Place an initial loading message in the popup
		var loadingDiv = dojo.create("div", null, linksDiv);
		loadingDiv.appendChild(document.createTextNode(this._messages.loading));
		
		this._getLinkMapForResource(this.concept).then(dojo.hitch(this, function(linkTypeURIMap) {
			dojo.style(loadingDiv, "display", "none");
			
			// Note that we sort the keys first. This is for consistency in the order of the output.
			var linkTypeKeys = Object.keys(linkTypeURIMap).sort();
			for (var i = 0; i < linkTypeKeys.length; i++) {
				var linkType = linkTypeKeys[i];
				var entry = linkTypeURIMap[linkType],
					a2bTypeLabelAdded = false,
					b2aTypeLabelAdded = false;
				if (entry.a2bLinks) {
					var linkTypeTable,
						resourceArr = this._getSortedArrayFromLinkTypeEntries(entry.a2bLinks);
					for (var j = 0; j < resourceArr.length; j++) {
						var resourceEntry =  resourceArr[j];
						if (resourceEntry.validityEntry.excludedFromRollup) {
							continue;
						}
						var row = this._getLinkRowForHover(linkType, resourceEntry);
						if (row) {
							// We know we have at least one entry for the link type. Add the link type label and create the entry table if we have not already done so.
							if (!a2bTypeLabelAdded) {
								a2bTypeLabelAdded = true;
								var sectionDiv = dojo.create("div", {"class": "link-type-heading"}, linksDiv);
								sectionDiv.appendChild(document.createTextNode(entry.a2bName+":"));
								linkTypeTable = dojo.create("table", {"class": "link-table"}, linksDiv);
							}
							dojo.place(row, linkTypeTable);
						}
					}
				}
				// Now do the same thing for the b2a links
				if (entry.b2aLinks) {
					var linkTypeTable,
						resourceArr = this._getSortedArrayFromLinkTypeEntries(entry.b2aLinks);
					for (var j = 0; j < resourceArr.length; j++) {
						var resourceEntry =  resourceArr[j];
						if (resourceEntry.validityEntry.excludedFromRollup) {
							continue;
						}
						var row = this._getLinkRowForHover(linkType, resourceEntry);
						if (row) {
							if (!b2aTypeLabelAdded) {
								b2aTypeLabelAdded = true;
								var sectionDiv = dojo.create("div", {"class": "link-type-heading"}, linksDiv);
								sectionDiv.appendChild(document.createTextNode(entry.b2aName+":"));
								linkTypeTable = dojo.create("table", {"class": "link-table"}, linksDiv);
							}
							dojo.place(row, linkTypeTable);
						}
					}
				}
			}
		}));
		return container;
	},
	
	_getLinkMapForResource: function(conceptUri) {
		// summary: Creates a map of the links for a given resource
		//	this method is based heavily on some logic found in compareLinksUtils.js,
		//	but simplified as we do not want to (nor can we) compute comparison information.
		//
		//		returns: An object formatted as such
		//		{
		//			linkType1URI : 
		//			{
		//				"a2bLinks" : 
		//				{
		//					linkAURI : <resource entry>,
		//					linkBURI : <resource entry>
		//				},
		//				"a2bName"  : "Link To",
		//				"b2aLinks" : 
		//				{
		//					linkCURI: <resource entry>
		//				},
		//				"b2aName" : "Link From"
		//			}
		
		var deferred = new dojo.Deferred();
		var linksDeferred = ServerLinkUtils.getLinksForResource({
			sourceAndTargetURI: conceptUri,
			skipCache: true
		});
		linksDeferred.addCallback(this, function(args) {
			var links = args.resourceInfos,
				linkTypeURIMap = {},
				resourcesToGet = [],
				typeUris = [],
				resourceName;
			
			// Get the list of link type URIs
			dojo.forEach(links, dojo.hitch(this, function(link) {
				typeUris.push(link._linkType);
			}));
			
			var linkTypesDeferred = LinkTypeUtils.getLinkTypes(typeUris);
			linkTypesDeferred.addCallback(this, function(types) {
				
				// Process the types in order to determine the link names, and their direction
				dojo.forEach(links, dojo.hitch(this, function(link) {
					var linkName,
						linkType,
						typeUri;
					// Associate the type object relating to the link we're dealing with
					var linkType = dojo.filter(types, function (type) {
						return type.getResourceURI() === link._linkType;
					});
					// Since that filter returned an array, extract the object
					linkType = linkType[0];
					if (!linkType) {
						return;
					}
					
					typeUri = linkType.getSameAs() ? linkType.getSameAs() : linkType.getResourceURI();
					if (!linkTypeURIMap[typeUri]) {
						linkTypeURIMap[typeUri] = {};
					}
										
					// This is the subject, so add it into the "a2bLinks" area and populate "a2bName"
					if (linkType && this.concept === link._subject) {
						// Create our object if we don't have it yet, but this "a2bLinks"
						// object is only being created because we plan to put a resource
						// into it.  Otherwise we don't want this field.  Same applies to
						// "b2aLinks"
						if (!linkTypeURIMap[typeUri]["a2bLinks"]) {
							linkTypeURIMap[typeUri]["a2bLinks"] = {};
						}
						
						resourceName = link._object;
						
						// Match the appropriate details entry
						var detailsArr = this._getEntry() ? this._getEntry().validityDetails : [],
							validityEntry = null;
						for (var i = 0; i < detailsArr.length; i++) {
							var detailsEntry = detailsArr[i];
							if (detailsEntry.linkType === typeUri
									&& detailsEntry.srcConcept === this.concept
									&& detailsEntry.targetConcept === resourceName) {
								validityEntry = detailsEntry;
							}
						}
						linkTypeURIMap[typeUri]["a2bLinks"][resourceName] = {"uri": resourceName, "title" : resourceName, "id" : link._title, validityEntry: validityEntry};
						linkTypeURIMap[typeUri]["a2bName"] = linkType.getSubjectToObjectLabel();
					} else if (linkType) {
						// Not the subject, so it's the object.  Otherwise do the same things.
						if (!linkTypeURIMap[typeUri]["b2aLinks"]) {
							linkTypeURIMap[typeUri]["b2aLinks"] = {};
						}
						
						resourceName = link._subject;
						
						// Match the appropriate details entry
						var detailsArr = this._getEntry() ? this._getEntry().validityDetails : [],
							validityEntry = null;
						for (var i = 0; i < detailsArr.length; i++) {
							var detailsEntry = detailsArr[i];
							if (detailsEntry.linkType === typeUri
									&& detailsEntry.srcConcept === resourceName
									&& detailsEntry.targetConcept === this.concept) {
								validityEntry = detailsEntry;
							}
						}
						linkTypeURIMap[typeUri]["b2aLinks"][resourceName] = {"uri": resourceName, "title" : resourceName, "id" : link._title, validityEntry: validityEntry};
						linkTypeURIMap[typeUri]["b2aName"] = linkType.getObjectToSubjectLabel();
					}
					// RM internal link, so add it to the queue of resources to be processed
					if (LinkUtils.getURLInfo(resourceName).sameLocation) {
						resourcesToGet.push({"resourceURI" : resourceName});
					}
				}));
	
				// Get our resources if we have any to get
				if(resourcesToGet.length) {
					ResourceManager.getMultiple(resourcesToGet, dojo.hitch(this, function(resources) {
						
						// Go through each resource and put it in its proper place
						dojo.forEach(resources, function(resource) {
							if (!resource.getID) {
								return;
							}
						
							var nameInfo = {"id" : resource.getID(), "title" : resource.getTitle(), "resource": resource};
							for (var linkType in linkTypeURIMap) {
								if (linkTypeURIMap[linkType]["a2bLinks"] && 
									linkTypeURIMap[linkType]["a2bLinks"][resource.resourceURI]) {
									linkTypeURIMap[linkType]["a2bLinks"][resource.resourceURI] = dojo.mixin(linkTypeURIMap[linkType]["a2bLinks"][resource.resourceURI], nameInfo);
								}
								
								if (linkTypeURIMap[linkType]["b2aLinks"] &&
									linkTypeURIMap[linkType]["b2aLinks"][resource.resourceURI]) {
									linkTypeURIMap[linkType]["b2aLinks"][resource.resourceURI] = dojo.mixin(linkTypeURIMap[linkType]["b2aLinks"][resource.resourceURI], nameInfo);
								}
							}
						});
						deferred.callback(linkTypeURIMap);
					}));
				} else {
					// Just fire our callback with the URI map.  
					deferred.callback(linkTypeURIMap);
				}
			});
		});
		return deferred;
	},
	
	_getSortedArrayFromLinkTypeEntries: function(linkTypeEntry) {
		// Put the entries into an array and sort them by status
		// Note that this is ugly, because we can't use a typical sort function for this because
		//	we want to return the entries by status, and the order does not correspond
		//	conveniently with the enumerated status values.
		var resourceKeys = Object.keys(linkTypeEntry),
			suspectArr = [],
			invalidArr = [],
			validArr = [],
			otherArr = [];
		for (var i = 0; i < resourceKeys.length; i++) {
			var entry = linkTypeEntry[resourceKeys[i]];
			if (!entry.validityEntry) {
				// The entry may not be for a valid link type.
				continue;
			}
			switch (entry.validityEntry.status) {
				case LinkValidityUtils.LINK_STATUS.VALID:
					validArr.push(entry);
					break;
				case LinkValidityUtils.LINK_STATUS.INVALID:
					invalidArr.push(entry);
					break;
				case LinkValidityUtils.LINK_STATUS.UNKNOWN:
					suspectArr.push(entry);
					break;
				default:
					otherArr.push(entry);
			}
		};
		return invalidArr.concat(suspectArr, validArr, otherArr);
	},
	
	_getLinkRowForHover: function(linkTypeUri, linkEntry) {
		// summary: Finds the matching validity details for the provided link entry
		//	returns: The row for the rich hover with the link and icon if the link
		//		was found and it had a matching validity entry in this widgets entry details.
		//		Otherwise, returns null.
		if (!linkEntry || !linkEntry.uri) {
			return null;
		}
        
		if (linkEntry.validityEntry) {
			// We have found the details entry that corresponds to the provided linkEntry.
			//	Decorate the link accordingly.
			var	title,
				iconClass;
			switch (linkEntry.validityEntry.status) {
				case LinkValidityUtils.LINK_STATUS.VALID:
					title = this._messages.valid;
					if (LinkValidityProfile.getDownstreamLinkTypes()[linkEntry.validityEntry.linkType])
				        iconClass = "com-ibm-rdm-web-linkvalidity-valid-hollow-icon";
                    else
                        iconClass = "com-ibm-rdm-web-linkvalidity-valid-icon";
					break;
				case LinkValidityUtils.LINK_STATUS.INVALID:
					title = this._messages.invalid;
					if (LinkValidityProfile.getDownstreamLinkTypes()[linkEntry.validityEntry.linkType])
				        iconClass = "com-ibm-rdm-web-linkvalidity-invalid-hollow-icon";
                    else
                        iconClass = "com-ibm-rdm-web-linkvalidity-invalid-icon";
					break;
				case LinkValidityUtils.LINK_STATUS.UNKNOWN:
					title = this._messages.unknown;
					if (LinkValidityProfile.getDownstreamLinkTypes()[linkEntry.validityEntry.linkType])
				        iconClass = "com-ibm-rdm-web-linkvalidity-unknown-hollow-icon";
                    else
                        iconClass = "com-ibm-rdm-web-linkvalidity-unknown-icon";
					break;
				default:
					// No status.
					return null;
			}
			
			var	row = dojo.create("tr", null);
			var iconCell = dojo.create("td", null, row);
			var icon = dojo.create("div", {"style": "display: block; width: 17px;"}, iconCell);
			dojo.attr(icon, "title", title);
			dojo.addClass(icon, iconClass);
			
			// Create the label for the link, then create the link itself
			var uri,
				label;
			var artifactIconCell = dojo.create("td", null, row);
			var iconNode = dojo.create("div", {"class": "icon-node"}, artifactIconCell);
			var linkCell = dojo.create("td", null, row);
			var linkNode = dojo.create("a", null, linkCell);
			
			if (!linkEntry.resource) {
				// External link
				uri = linkEntry.uri;
				label = linkEntry.uri;
				
			} else if (linkEntry.resource.getID() && linkEntry.resource.getTitle()) {
				uri = VvcUtils.getDefaultLink(linkEntry.uri);
				label = linkEntry.resource.getID() + ":" + linkEntry.resource.getTitle();
			}
			
			var resourceLink = new ResourceLinkNoHover({
				uri: uri,
				label: label,
				iconNode: iconNode,
				lazyFetch: false
			}, linkNode);
			
			return row;
		}
		return null;
	}
	
});
})();
