/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() { 
dojo.provide("rmps.app.ui.internal.linkvalidity.LinkValidityStatusComboButton");

dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.store.Memory");
dojo.require("rmps.app.ui.internal.linkvalidity.LinkValidityUtils");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.linkvalidity", "LinkValidityControlWidget");	

var LinkValidityUtils = rmps.app.ui.internal.linkvalidity.LinkValidityUtils,
	FilteringSelect = dijit.form.FilteringSelect,
	Memory = dojo.store.Memory;


dojo.declare("rmps.app.ui.internal.linkvalidity.LinkValidityStatusComboButton", [dijit._Widget], {
	
	showDoNotChangeOption: false,
	status: LinkValidityUtils.LINK_STATUS.VALID,
	
	_iconNode: null,
	_filteringSelect: null,
	
	postMixInProperties: function() {
		this.inherited("postMixInProperties", arguments);
		this.messages = dojo.i18n.getLocalization("rmps.app.ui.internal.linkvalidity", "LinkValidityControlWidget");
	},
	
	postCreate: function() {
		// Create the status combo box
		var data = [];
		
		if (this.showDoNotChangeOption) {
			data.push({name: this.messages.doNotChange, id: LinkValidityUtils.LINK_STATUS.DO_NOT_CHANGE, label: this._getLabel("com-ibm-rdm-web-linkvalidity-blank-icon", this.messages.doNotChange)});
		}
		data = data.concat([
            {name: this.messages.valid, id: LinkValidityUtils.LINK_STATUS.VALID, label: this._getLabel("com-ibm-rdm-web-linkvalidity-valid-icon", this.messages.valid)},
            {name: this.messages.invalid, id: LinkValidityUtils.LINK_STATUS.INVALID, label: this._getLabel("com-ibm-rdm-web-linkvalidity-invalid-icon", this.messages.invalid)},
            {name: this.messages.unknown, id: LinkValidityUtils.LINK_STATUS.UNKNOWN, label: this._getLabel("com-ibm-rdm-web-linkvalidity-unknown-icon", this.messages.unknown)}
        ]);
		var stateStore = new Memory({
			data: data
		});
		 
		this.domNode = dojo.create("div");
		var contentTable = dojo.create("table", null, this.domNode);
		var row = dojo.create("tr", null, contentTable);
		var iconCell = dojo.create("td", null, row),
			comboBoxCell = dojo.create("td", null, row);
		 
		// Create the iconNode, then set the initial state
		this._iconNode = dojo.create("div", {style: "vertical-align: middle; margin-right: 2px;"}, iconCell);
		this._selectStatus(this.status);
		 
		// Create the combo box
		this._filteringSelect = new FilteringSelect({
				store: stateStore,
				value: this.status,
				labelAttr: "label",
				labelType: "html"
			});
		this.connect(this._filteringSelect, "onChange", dojo.hitch(this, function() {
			this._selectStatus(this._filteringSelect.value);
		}));
		this._filteringSelect.placeAt(comboBoxCell).startup();
	},
	
	getStatus: function() {
		return this.status;
	},
	
	focus: function() {
		// Gives focus to the FilteringSelect widget.
		this._filteringSelect.focus();
	},
	
	_getLabel: function(className, labelName) {
		return "<table><tr><td  class='" + className + "'></td><td>" + labelName + "</td></tr></table>";
	},
	
	_selectStatus: function(status) {
		// Set the status and update the icon
		this.status = status;
		var imgClass = "";
	 	switch (status) {
	 	case LinkValidityUtils.LINK_STATUS.DO_NOT_CHANGE:
				imgClass = "com-ibm-rdm-web-linkvalidity-blank-icon";
				break;
			case LinkValidityUtils.LINK_STATUS.VALID:
				imgClass = "com-ibm-rdm-web-linkvalidity-valid-icon";
				break;
			case LinkValidityUtils.LINK_STATUS.INVALID:
				imgClass = "com-ibm-rdm-web-linkvalidity-invalid-icon";
				break;
			case LinkValidityUtils.LINK_STATUS.UNKNOWN:
				imgClass = "com-ibm-rdm-web-linkvalidity-unknown-icon";
				break;
			default:
				imgClass = "com-ibm-rdm-web-linkvalidity-valid-icon";
		}
		dojo.removeClass(this._iconNode);
		dojo.addClass(this._iconNode, imgClass);
	}
	
});
})();
