/*
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
 * 
 * (c) Copyright IBM Corporation 2015, 2016.  
 * U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
	dojo.provide("rmps.app.ui.internal.pages.DmChangeSetEditor");

	dojo.require("dojo.i18n");
	dojo.require("com.ibm.team.repository.web.configurationManagement.editors.CmChangeSetEditor");
	dojo.require("com.ibm.team.repository.web.configurationManagement.CmButtonFactory");
	dojo.require("com.ibm.team.repository.web.configurationManagement.editors.CmTabManager");
	dojo.require("com.ibm.team.repository.web.configurationManagement.grid.CmTreeGridTable");
	dojo.require("com.ibm.team.repository.web.configurationManagement.CmConstants");
	dojo.require("com.ibm.team.repository.web.configurationManagement.CmEvents");
	
	dojo.require("rmps.app.ui.internal.changesets.ChangeSetResourcesTable");
	var ChangeSetResourcesTable = rmps.app.ui.internal.changesets.ChangeSetResourcesTable;
	
	dojo.require("rmps.app.ui.internal.util.DmButtonFactory");
	var DmButtonFactory = rmps.app.ui.internal.util.DmButtonFactory;
	
	dojo.require("rmps.app.ui.internal.changesets.ChangeSetLinksTable");
	var ChangeSetLinksTable = rmps.app.ui.internal.changesets.ChangeSetLinksTable;

	var CmChangeSetEditor = com.ibm.team.repository.web.configurationManagement.editors.CmChangeSetEditor;
	var CmButtonFactory = com.ibm.team.repository.web.configurationManagement.CmButtonFactory;
	var Tab = com.ibm.team.repository.web.configurationManagement.editors.CmTab;

	var CmConstants = com.ibm.team.repository.web.configurationManagement.CmConstants;
	var CmEvents = com.ibm.team.repository.web.configurationManagement.CmEvents;

	dojo.requireLocalization("rmps.app.ui.internal.pages", "DmChangeSetEditorMessages");
    var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DmChangeSetEditorMessages");
	// Summary:
	//      Provides the ChangeSetEditor for RM.  This is a subclass of the foundation
	//      provided CmChangeSetEditor.  This subclass adds the following capabilities
	// 
	//      1. Puts the related RM project name and banner button into the Banner, this comes
	//         from the _RmConfigurationEditorMixin.      
	//
	//      2. Add Link, Complete, Deliver, Discard and Compare buttons to the toolbar in the upper right.
	//    
	//      3. Add in the banner button and the RM specific menus.
	//
	//      4. Provide a single tab that displays any Work Item links and provides
	//         actions to add and remove links.

	dojo.declare("rmps.app.ui.internal.pages.DmChangeSetEditor", 
			[CmChangeSetEditor], {

		// the action registered with foundation Web UI, registration is performed
		// in CmEditor
		editorAction: "rmps.app.editChangeSet",
		linksTable: null,

		postCreate: function() {
			// Summary:
			//      Initialise the RM Web UI so that we can use RM services to fetch 
			//      links etc.  

//			dojo.addClass(this.domNode, "com-ibm-rdm-web-pages-RmChangeSetEditor");

//			com.ibm.rdm.web.pages.WebUIinitializer.init(jazz.app.currentApplication);
			this.inherited(arguments);

			// this event is generated when the user e.g. completes the change set via the
			// banner button.  This is actually the only case we are interested in, refresh
			// so that the buttons are updated.
			//
			// Since actions in the banner button may affect the state of this change set
			// then we need to refresh the UI in order to pick those up.
			//dojo.subscribe(CmEvents.USER_REQUESTED_CONTEXT_CHANGE, this, this.refresh);
		},
		
//		_getIconClass: function _getIconClass_$2() {
//            // Summary:
//            //      Override this method to produce the icon in the top left.
//            //      Ppick the icon based on the state of the change set, the
//            //      superclass otherwise just uses a generic change set icon.
//            
//            return "changeSetIcon-" + this.resource.state;
//        },

		_buildTabs: function __buildTabs(){
			var tabs = this.inherited(arguments);
			tabs.push(this.buildLinksTab());
			tabs.push(this.buildResourcesTab());
			return tabs;
		},

		buildLinksTab: function __buildLinksTab(){
			var initialiser = dojo.hitch(this, function __linksTabInitializer(){
				this.linksTable = ChangeSetLinksTable({
					changesetUri: this.resourceUri
				});
				this.linksTable.refresh();
				return this.linksTable.domNode;
			});

			var tab = new Tab({
				id: "links",
				title: messages.linksTabTitle,
				helpContent: messages.linksTabHelp,
				index: 1,
				initialiser: initialiser
			});
			return tab;
		},

		buildResourcesTab: function __buildResourcesTab(){
			var initialiser = dojo.hitch(this, function __resourcesTabInitializer(){
				var resourcesTable = ChangeSetResourcesTable({
					changesetUri: this.resourceUri
				});
				resourcesTable.refresh();
				return resourcesTable.domNode;
			});

			var tab = new Tab({
				id: "resources",
				title: messages.resourcesTabTitle,
				helpContent: messages.artifactsTabHelp,
				index: 0,
				initialiser: initialiser
			});
			return tab;
		},
				
		buildActions: function buildActions() {
            // Summmary:
            //      Override of base class method.  This functions responsibility is
            //      to return an array of buttons.  It appears these ought to be of type
            //      jazz.ui.toolbar.Button.  The functions we want are already provided
            //      by buttons in the CmButtonFactory class, so we jsut need to get them
            //      and customise them where apporopriate.
            //
            // returns:
            //      An array of jazz.ui.toolbar.Buttons
            
            // get any buttons the super classes want to add first...
            var actions = this.inherited(arguments),
                compareWithStreamBtn,
                deliverBtn,
                discardBtn,
             // completeBtn,
                addLinkBtn,
                addLinkCallback = dojo.hitch(this, function(links) {
                    if(this.linksTable !== null && this.linksTable._gridTable) {
	                    this.linksTable._gridTable.addRow(links);
	                    dojo.removeClass(this.domNode, "noLinks");
	                }
                });
                
            compareWithStreamBtn = CmButtonFactory.getCompareChangesetWithStreamButton(this.resource, {
                disabled: false
            }, dojo.hitch(this, function() {
                return {
                    sourceUri: this.resource.id,
                    targetUri: this.resource.parentStreamUri
                };
            }));
            
            discardBtn = CmButtonFactory.getDiscardChangesetButton({
                   id: this.resource.id,
                   state: this.resource.state 
                }, {
                    okCallback: dojo.hitch(this, function() {
                        // redirect back to the parent stream, not sure what else we could
                        // do here.
                        window.location.hash = "#action=rmps.app.editStream&configurationUri=" + encodeURI(this.resource.parentStreamUri);
                })
            });
           
            deliverBtn = DmButtonFactory.getChangeSetDeliverButton(null, {
                disabled: false
            }, dojo.hitch(this, function() {
                return {
                    sourceUri: this.resource.id,
                    targetUri: this.resource.parentStreamUri
                };
            }));
            
            if (this.resource.state === CmConstants.CHANGE_SET_STATES.COMMITTED) {
                deliverBtn.set("enabled", false);
            }
            
            if (this.resource.state === CmConstants.CHANGE_SET_STATES.COMMITTED) {
                compareWithStreamBtn.set("enabled", false);
            }

            addLinkBtn = DmButtonFactory.getLinkToWorkItemButton(this.resource, addLinkCallback);
            
            actions.unshift(compareWithStreamBtn);
            actions.unshift(discardBtn);
            actions.unshift(deliverBtn);
            actions.unshift(addLinkBtn);
            
            this._gc.destroys(deliverBtn);
            this._gc.destroys(compareWithStreamBtn);
            this._gc.destroys(discardBtn);
            this._gc.destroys(addLinkBtn);
            
            return actions;
        }
	});
}());