/*
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
 * 
 * (c) Copyright IBM Corporation 2015, 2016.  
 * U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
    dojo.provide("rmps.app.ui.internal.pages.DmStreamEditor");
    
    dojo.require("rmps.app.ui.internal.util.DmTabsBuilder");
    var DmTabsBuilder = rmps.app.ui.internal.util.DmTabsBuilder;
    
    dojo.require("com.ibm.team.repository.web.configurationManagement.editors.CmTabManager");
    var Tab = com.ibm.team.repository.web.configurationManagement.editors.CmTab;
    
    dojo.require("rmps.app.ui.internal.changesets.DmChangeSetsExplorer");
    var DmChangeSetsExplorer = rmps.app.ui.internal.changesets.DmChangeSetsExplorer;
    
    dojo.requireLocalization("rmps.app.ui.internal.pages", "DmStreamEditorMessages");
    var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DmStreamEditorMessages");
    
    dojo.requireLocalization("com.ibm.team.repository.web.configurationManagement.editors", "CmEditorMessages");
    var CmMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.configurationManagement.editors", "CmEditorMessages");
    
    dojo.require("rmps.app.ui.internal.util.DmButtonFactory");
    var DmButtonFactory = rmps.app.ui.internal.util.DmButtonFactory;	
    
    dojo.require("com.ibm.team.repository.web.configurationManagement.editors.CmStreamEditor");
//    dojo.require("com.ibm.rdm.web.pages._RmConfigurationEditorMixin");
    var CmStreamEditor = com.ibm.team.repository.web.configurationManagement.editors.CmStreamEditor;
  //  var _RmConfigurationEditorMixin = com.ibm.rdm.web.pages._RmConfigurationEditorMixin;
   
    // Summary:
    //      This is a simple subclass of the foundation Stream editor. Just adds the project name
    //      and context button to the banner.
    dojo.declare("rmps.app.ui.internal.pages.DmStreamEditor", 
                    [CmStreamEditor], {

         // CmEditor will register this Action with the ActionRegistry...needs to match
         // the entry in com.ibm.rdm.web/plugin.xml                     
         editorAction: "rmps.app.editStream",
         workspacesViewer: null,
         
		_buildTabs: function __buildTabs(){
			var tabs = this.inherited(arguments);
			tabs.push(DmTabsBuilder.buildDependenciesTab(this, true));
			return tabs;
		},
		
		buildActions: function() {
			var actions = this.inherited(arguments);
			for (var i = 0; i < actions.length; i++) {
				var action = actions[i];
				if (action) {
					if (action.imgClass === "icons-compare-with-flow-target") {
						var dmAction = DmButtonFactory.getCompareButton(null, {
							disabled: false,
							imgClass: action.imgClass,
							title: action.title
						}, dojo.hitch(this, function __getCompareButton(){
							return {
								sourceUri: this.resource.id,
								targetUri: this.resource.parentStreamUri
							};
						}));
						actions.splice(i, 1, dmAction);
					} else if (action && action.imgClass === "icons-accept") {
						var dmAction = DmButtonFactory.getAcceptButton(null, {
							disabled: false,
							imgClass: action.imgClass,
							title: action.title
						}, dojo.hitch(this, function __getAcceptButton(){
							return {
								targetUri: this.resource.id,
								sourceUri: this.resource.parentStreamUri
							};
						}));
						actions.splice(i, 1, dmAction);
					} else if (action && action.imgClass === "icons-deliver") {
						var dmAction = DmButtonFactory.getDeliverButton(null, {
							disabled: false,
							imgClass: action.imgClass,
							title: action.title
						}, dojo.hitch(this, function __getDeliverButton(){
							return {
								sourceUri: this.resource.id,
								targetUri: this.resource.parentStreamUri
							};
						}));
						actions.splice(i, 1, dmAction);
					}
				}
			}
			return actions;
		},
		
		//override the buildChangesetsTab in com.ibm.team.repository.web.configurationManagement.editors.CmStreamEditor
		buildChangesetsTab: function __buildChangesetsTab(){	
			var initialiser = dojo.hitch(this, function(){
				var changesetsExplorer = new DmChangeSetsExplorer({
					configurationUri: this.resourceUri
				});
				return changesetsExplorer;
			});
			
			var tab = new Tab({
				id: "changesets",
				title: CmMessages.changesetsTabTitle,
				helpContent: CmMessages.changesetsTabHelpContent,
				index: 0,
				initialiser: initialiser
			});
			return tab;
		}
		
    });
}());