/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2015. All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.MinibarCellSidebar");

dojo.require("rmps.ui.SidebarContainer");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("rmps.ui.explorer.ExplorerSidebarWidget");

dojo.declare("rmps.app.ui.internal.pages.MinibarCellSidebar", [dijit._Widget, dijit._Container], {
	/*mixin*/
	pageTemplate: null,

	_sidebar: null,
	_explorer: null,
	
	baseClass: "rmps-app-ui-internal-pages-MinibarCellSidebar-container",

	postCreate: function() {
		this.inherited(arguments);
		
		this.subscribe("rmps.app.ui.internal.pages.DisplaySidebar", this.displaySidebar);
	},
	
	_createExplorer: function() {
		this._explorer = new rmps.ui.explorer.ExplorerSidebarWidget({
			explorerID: rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE
		});

		var viewport = dijit.getViewport();
		this._sidebar = new rmps.ui.SidebarContainer({
			onUpdateWidth: dojo.hitch(this, function() {
				this._checkHeight(true);
				this._updateWidth();
			}),
			sidebarContainerWidth: viewport.w
		});
		this._sidebar.addSidebar(this._explorer, true /*left*/, false /*isOpen*/, true /*preserveState*/, false /*isStationary*/);
			
		this.scrollHandler = this.connect(window, "onscroll", "_onTrack");
		this.resizeHandler = this.connect(window, "onresize", "_onTrack");
		this.resizeNotHandler = jazz.util.ResizeNotifier.register({context: this, callback: this._onTrack, force: true});
		
		this.addChild(this._sidebar);
		
		this.subscribe("rmps.app.ui.internal.pages.ShowSidebar", this.showSidebar);
	},
	
	display: function(display) {
		if (display && !this._explorer) {
			console.debug("Creating explorer");
			this._createExplorer();
			this._checkHeight(false);
			this._updateWidth();
		}
		console.debug("Explorer displayed set to " + display);
		dojo.style(this.domNode.parentNode, "display", display ? "" : "none");
	},
	
	_onTrack: function() {
		this._checkHeight(false);
	},
	
	_checkHeight: function(force) {
		var viewport = dijit.getViewport();
		var navPos = this.pageTemplate._navbarWrapper ? dojo.position(this.pageTemplate._navbarWrapper) : {y: 0, h: 0};
		var footerPos = this.pageTemplate._footerWrapper ? dojo.position(this.pageTemplate._footerWrapper) : {y: 0, h: viewport.h};
		var height;
		var node = this._sidebar.domNode;
		if (navPos.y <= 0) {
			if (!this._fixed||force===true) {
				dojo.style(node, "position", "fixed");
				dojo.style(node, "top", navPos.h + 2 + "px");
				this._fixed = true;
			}
			height = viewport.h - navPos.h - 8;
		} else {
			if (this._fixed||force===true) {
				dojo.style(node, "position", "absolute");
				dojo.style(node, "top", "");
				this._fixed = false;
			}
			height = viewport.h - (navPos.y + navPos.h) - 7;
		}
		if (viewport.h > footerPos.y) {
			height = height - (viewport.h - footerPos.y);
		}
		height = Math.floor(height);
		if (force || (height !== this._height && height >= 0)) {
			console.debug("Setting sidebar height to " + height);
			dojo.style(node, "height", height + "px");
			this._explorer.resize({h:height});
			this._sidebar.resize({h:height});
			this._height = height;
		}
	},
	
	_updateWidth: function() {
		var sidebar = this._sidebar ? this._sidebar._sidebars[0] : null;
		var width = 0;
		if (sidebar && sidebar.isOpen) {
			width = sidebar._sidebarWidth;
		}
		if (this._width !== width) {
			console.debug("Setting sidebar width to " + width);
			dojo.style(this.domNode, "width", width + "px");
			this._width = width;
		}
	},
	
	showSidebar: function(message) {
		if (message.explorerID !== rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE) {
			return;
		}
		
		if (!this._sidebar.openSidebars(true)){
			dojo.publish("rmps.ui.explorer.Explorer.loaded", [{
				explorerID: message.explorerID
			}]);
		}
	},
	
	displaySidebar: function(message) {
		if (message.explorerID !== rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE) {
			return;
		}
		
		this.display(message.display);
	}
});

}());