/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2015.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.ui.internal.pages.ReviewPage");

dojo.require("dm.review.app.ui.internal.pages.ReviewWelcome");
dojo.require("rmps.app.ui.internal.pages.SideNavigation");
dojo.require("rmps.app.ui.internal.ResourceEditorInput");
dojo.require("rmps.app.ui.ReviewEditorUtil");
dojo.require("dm.review.app.util.ReviewUtil");
dojo.require("rmps.ui.editor.EditorRegistry");
dojo.require("rmps.app.ui.internal.search.SearchBanner");
dojo.require("rmps.ui.editor.AbstractPage");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("rmps.core.review.util.ReviewUtil");

dojo.require("rmps.ui.editor.PageHelper");
dojo.require("dojo.i18n");

dojo.require("dm.review.app.ui.internal.editors.RegisterReviewEditorsInitializer");

dojo.requireLocalization("dm.review.app.ui.internal.pages", "ReviewWelcomeMessages");
var messages = dojo.i18n.getLocalization("dm.review.app.ui.internal.pages", "ReviewWelcomeMessages");

var PAGE_REMOVAL_ID = "com.ibm.xtools.rmps.jfs.web.removeReviewsPage";
var ACTIONS_LIST= ["_default", "welcome","newReviewEditor","reviewViewerEditor","viewResource"];
var PAGE_ID = dm.review.app.util.ReviewUtil.PAGE_ID;
var REVIEW_EDITOR_TYPE = rmps.app.ui.ReviewEditorUtil.REVIEW_EDITOR;
var RES_REVIEW_PREFIX = rmps.app.ui.ReviewEditorUtil.RES_REVIEW_PREFIX;
var ReviewUtil= rmps.core.review.util.ReviewUtil;

dojo.declare("dm.review.app.ui.internal.pages.ReviewPage", rmps.ui.editor.AbstractPage, {

	// private properties
	_welcome: null,
	_previousEditorView: null,
	_reviewViewerEditorInput: null,
	
	_defaultUrl: null,
	
	createPageContents: function(){
		if (this._reviewViewerEditorInput) {
			this.inherited(arguments);
			return;
		}
		
		var editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(rmps.app.ui.ReviewEditorUtil.REVIEW_EDITOR_VIEWER);
		editorInputDeferred.then(dojo.hitch(this, function(editorInput) {
			this._reviewViewerEditorInput = editorInput;
						
			this.createPageContents();
		}));
	},
	
	getPageRemovalId: function(){
		return PAGE_REMOVAL_ID;
	},
	
	subscribeActions: function(){
		this.subscribeAllActions(this);	
	},
	
	registerActions: function(){
		this.registerActionsList(ACTIONS_LIST);
	},
	
	getPageId: function(){
		return PAGE_ID;
	},
		
	/*FIXME: Copied from ModelingPage. We should have an abstract page to have the common methods*/
	viewResource: function(args) {
		if(!args){
			return;
		}
		var resourceUri= args.resourceUri;
		
		this.displaySidebar(true);
		
		var editorInput = null;
		if (args.clone){
			editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(rmps.app.ui.ReviewEditorUtil.REVIEW_EDITOR, {
				resourceUri: resourceUri + "&" + ReviewUtil.CLONE_PARAM,
				id: "cloneReview",
				ts: new Date().getTime()
			});
		} else if (args.type) {
			editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(args.type, args);
		} else if (resourceUri.indexOf(RES_REVIEW_PREFIX)>-1){
			editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(REVIEW_EDITOR_TYPE, args);
		} else {
			editorInput = new rmps.app.ui.internal.ResourceEditorInput(args);
		}
		
		if (editorInputDeferred) {
			editorInputDeferred.then(dojo.hitch(this, function(editorInput) {
				editorInput.setParams(args);
				this.pageHelper.openEditor(editorInput);
			}));
		}
		else if (editorInput) {
			editorInput.setParams(args);
			this.pageHelper.openEditor(editorInput);
		}
		
	},

	newReviewEditor:function(args){
		if (this._defaultUrl === window.location.href) {
			return;
		}
		
		this.displaySidebar(true);
		
		var inputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(rmps.app.ui.ReviewEditorUtil.REVIEW_EDITOR, {
			resourceUri: rmps.app.ui.ReviewEditorUtil.NEW_REVIEW_EDITOR,
			id: "newReview",
			ts: new Date().getTime()
		});
		
		inputDeferred.then(dojo.hitch(this, function(input) {
			this.pageHelper.openEditor(input, args);
		}));
	},
		
	welcome: function() {
		this.pageHelper.hideActiveEditor();

		if (!this._welcome) {
			this._welcome = new dm.review.app.ui.internal.pages.ReviewWelcome();
			this.pageHelper.getEditorView().addChild(this._welcome);

			this._welcome.connect(this.pageHelper,"onEditorActivated", dojo.hitch(this, function () {
				this._welcome.domNode.style.display= 'none';
			}));

		}
		this._welcome.domNode.style.display= '';
		net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(messages.review_welcome_title_text);
		this._defaultUrl = window.location.href;
	},

	reviewViewerEditor: function() {
		this.displaySidebar(true);
		this.pageHelper.openEditor(this._reviewViewerEditorInput);
	},
	
	_getSearchBanner: function() {
		if (!this._searchBanner) {
			this._searchBanner = new rmps.app.ui.internal.search.SearchBanner();
			this._searchBanner.startup();
		}
		return this._searchBanner;
	},
		
	_default: function(args) {
		if (this._defaultUrl) {
			setTimeout(dojo.hitch(this, function() {
				window.location.href = this._defaultUrl;
			}), 0);
			return;
		}
		this.welcome();
	}
});


})();