/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {

	dojo.provide("rmps.app.ui.internal.pages.WorkspacesAndSnapshotsViewer");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated")
	dojo.require("dojo.date.stamp");
	dojo.require("dojo.date.locale");
	
	dojo.require("jazz.ui.MessageBox");
	dojo.require('jazz.util.html');
	dojo.require("jazz.ui.StyledTable");
		
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

//	dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils");
//	var SecurityServiceUtils = com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils;
	
	dojo.require("rmps.app.ui.internal.util.UserUiNode");
	var UserUiNode = rmps.app.ui.internal.util.UserUiNode;

	dojo.require("rmps.app.ui.internal.util.ConfigurationUiNode");
	var ConfigurationUiNode = rmps.app.ui.internal.util.ConfigurationUiNode;
	
	dojo.require("rmps.app.ui.internal.util.ProjectAreaUiNode");
	var ProjectAreaUiNode = rmps.app.ui.internal.util.ProjectAreaUiNode;

	dojo.require("rmps.app.ui.internal.util.InfoContentUtils");
	var InfoContentUtils = rmps.app.ui.internal.util.InfoContentUtils;
	
	dojo.require("rmps.app.ui.internal.util.ConfigurationResource");

	dojo.requireLocalization("rmps.app.ui.internal.pages", "SnapshotsViewerMessages");
	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "SnapshotsViewerMessages");
	
	var RMPS_APP = "rmps.app";
	var EDITABLE_LIST_STYLE = 0;
	var PICKER_STYLE = 1;
	var DEPENDENCIES_LIST_STYLE = 2;
	var DEPENDENCIES_LIST_WITHOUT_ACTIONS_STYLE = 3;
	var DEPENDENCY_SERVICE = "/dependencies";

	var NEW_WORKSPACE_ICON = dojo.moduleUrl(RMPS_APP, "graphics/icons/create_workspace-enabled.gif");
	var NEW_SNAPSHOT_ICON = dojo.moduleUrl(RMPS_APP, "graphics/icons/create_snapshot-enabled.gif");
	var COMPARE_ICON = dojo.moduleUrl(RMPS_APP, "graphics/icons/compare-enabled.gif");
	var DELETE_ICON = dojo.moduleUrl(RMPS_APP, "graphics/icons/delete.gif");

	dojo.declare("rmps.app.ui.internal.pages.WorkspacesAndSnapshotsViewer", [dijit._Widget, dijit._Templated], {
		messages: messages, // needed for the html template
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl(RMPS_APP, "ui/internal/pages/templates/WorkspacesAndSnapshotsViewer.html"),
		
		_configurationResources: null,
		_configResourcesInfo: null,
		_table: null,
		_style: null,
		_workspace: null,
		_workspaceUri: null,
		_configUris: null,
		_mutableIndex: 0,
		_lazyStart: false,

		constructor: function(args) {
			if (args) {
				this._style = args.style;
				this._workspaceUri = args.workspaceUri; //consider keeping workspace parameter only
				this._workspace = args.workspace;
				this._lazyStart = args.lazyStart;
			}
		},
		
		postCreate: function() {
			if (!this._lazyStart) {
				this.load();
			}
		},

		load: function() {
			// show loading message
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: this.messages.loading
			});
			
			var deferred;
			deferred = this.getConfigurationResources();
			deferred.addCallback(this, function (confResources) {
				this._configurationResources = confResources;
				this.rebuildTable(confResources);
			});
			deferred.addErrback(this, function(error) {
				rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
			});
		},
		
		refresh: function() {
			this._clear();
			this.load();
		},
		
		_clear: function() {
			if (this._table) {
				this._table.destroy();
				this._table = null;
			}
		},

		rebuildTable: function(configResources) {
			// remove loading message
			this.hideMessage();
			
			// create table
			var columnData = [];
			if (this._style == PICKER_STYLE) {
				columnData.push({ label: "", width: "16px" });  // check box
				this._mutableIndex = 1;
			}
			columnData.push({ label: messages.name, sortable: true, width: "250px" });
			columnData.push({ label: messages.description, sortable: true });
			columnData.push({ label: messages.parent, sortable: true });
			columnData.push({ label: messages.projectArea, sortable: true });
			columnData.push({ label: messages.creator, width: "150px", sortable: true  });
			columnData.push({ label: messages.created, width: "150px", sortable: true });
			if (this.isShowActionBar()) {
				columnData.push({ label: messages.actions, width: "75px" });
			}
			
			if (this._table) {
				this.tableContainerNode.removeChild(this._table);
			}
			var self = this;
			var table = new jazz.ui.StyledTable({
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				columns: columnData,
				sortBy: this._mutableIndex + 1, // name field
				sortByAscending: false,
				handleSort: function() {
					this.clearBody();
					self._sort(configResources, table);
					self._fillTable(configResources, table);
				}
			});
			this._table = table;
			this.tableContainerNode.appendChild(table.domNode);
			table.domNode.style.width = "100%";

			// mixed in the creator and created fields
			var userUri, userName;
			for (var i = 0; i < configResources.length; i++) {
				var configResource = configResources[i];

				configResource.mutable = configResource.getMutable();
				configResource.title = configResource.getTitle();
				configResource.description = configResource.getDescription();
				if (!configResource.description) {
					configResource.description = "";
				}
				configResource.parentName = configResource.getParentWorkspaceName();
				configResource.projectAreaName = configResource.getProjectAreaName();
				configResource.projectAreaUri = configResource.getProjectAreaUri();

				// set creator field
				configResource.creator = configResource.getCreator();
				var created = configResource.getCreated();			
				
				// set the created field
				if (created) {
					configResource.created = created;
				} else {
					configResource.created = "";
				}
			}

			this._sort(configResources, table);
			this._fillTable(configResources, table);
		},
		
		isShowActionBar: function() {
			if (this._style == EDITABLE_LIST_STYLE || this._style == DEPENDENCIES_LIST_STYLE) {
				return true;
			}
			return false;
		},

		_sort: function(configResources, table) {
			var tableColumn = this._getTableColumn;
			var mutableIndex = this._mutableIndex;
			var f = function(a, b) {
				var c1 = tableColumn(a, table.sortBy, mutableIndex);
				var c2 = tableColumn(b, table.sortBy, mutableIndex);
				var r = 0;
				if (c1 && c2) {
					if (c1 < c2) {
						r = -1; 
					} else if (c1 > c2) {
						r = 1;
					}
					if (table.sortByAscending) {
						r = r * -1;
					}
				} else if (c1) {
					return -1;
				} else {
					return 1;
				}
				return r;
			};
			configResources.sort(f);
		},

		_getTableColumn: function(configResource, columnIndex, index) {
			if (columnIndex === index++) {
				return configResource.mutable;
			}
			if (columnIndex === index++) {
				return configResource.title;
			}
			if (columnIndex === index++) {
				return configResource.description;
			}
			if (columnIndex === index++) {
				return configResource.parentName;
			}
			if (columnIndex === index++) {
				return configResource.projectAreaName;
			}
			if (columnIndex === index++) {
				return configResource.creator;
			}
			if (columnIndex === index++) {
				return configResource.created;
			}
		},

		_fillTable: function(configResources, table) {
			// Now add the rows
			var userUri, userName;
			var workspaceIcon = dojo.moduleUrl(RMPS_APP, "graphics/icons/workspace.gif");
			var snapshotIcon = dojo.moduleUrl(RMPS_APP, "graphics/icons/snapshot.gif");
			var td, tr;
			for (var i = 0; i < configResources.length; i++) {
				var configResource = configResources[i];
				tr = document.createElement("tr");

				// check box or radio button column
				if (this._style == PICKER_STYLE) {
					td = document.createElement("td");
					var buttonId = this.getCheckBoxId(i);
					td.innerHTML = "<input type='checkbox' id='" + buttonId + "'/>";
					tr.appendChild(td);
				}

				// add name column
				td = document.createElement("td");
				var configUri = configResource.uri;
				if (configUri) {
					var configUiNode = new ConfigurationUiNode({configResource: configResource}, td);
				} else {
					td.innerHTML = configResource.title;
				}
				tr.appendChild(td);

				// add description column
				td = document.createElement("td");
				td.innerHTML = jazz.util.html.escape(configResource.description);
				tr.appendChild(td);

				// add parent column
				td = document.createElement("td");
				var parentUri = configResource.getParentWorkspaceUri();
				if (parentUri) {
					var parentUiNode = new ConfigurationUiNode({uri: parentUri}, td);
				} else {
					td.innerHTML = configResource.parentName;
				}
				tr.appendChild(td);
				
				// add space column
				td = document.createElement("td");
				var spaceUiNode = new ProjectAreaUiNode({title: configResource.projectAreaName, uri: configResource.projectAreaUri}, td);
				tr.appendChild(td);
				
				// add creator column
				td = document.createElement("td");
				var userUiNode = new UserUiNode({uri: configResource.creator}, td);
				tr.appendChild(td);

				// add created date column
				td = document.createElement("td");
				var created = configResource.created;
				if (created) {
					var createdDate = dojo.date.stamp.fromISOString(created);
					created = dojo.date.locale.format(createdDate, {formatLength: "medium"});
				}
				td.innerHTML = created;
				tr.appendChild(td);

				// add actions column
				if (this.isShowActionBar()) {
					td = document.createElement("td");
					var tb = this.createActions(configResource, td);
					td.appendChild(tb.domNode);
					tr.appendChild(td);
				}

				table.tbody.appendChild(tr);
			}
			return table;
		},
		
		getCheckBoxId: function(index) {
			return "checkbox_" + index;
		},
		
		getSelected: function() {
			var selected = [];
			var configArray = this._configurationResources;
			if (configArray) {
				for (var i = 0; i < configArray.length; i++) {
					var buttonId = this.getCheckBoxId(i);
					var button = dojo.byId(buttonId);
					if (button && button.checked) {
						var uri = configArray[i].uri;
						selected.push(uri);
					}
				}
			}
			return selected;
		},

		_isMutable: function(configResource) {
			var mutable = configResource.getMutable();
			if (mutable === "true") {
				return true;
			}
			return false;
		},

		createActions: function(configResource, td) {
			var toolbar = new jazz.ui.Toolbar();
			var actions;
			if (this._style == DEPENDENCIES_LIST_STYLE) {
				var self = this;
				if(!this._workspace.permissions){
//					this._workspace.loadPermissionsSync(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "security/getAllPermissions?configurationUri=" + configResource.uri);
				}
				actions = [{//label: this._workspace.permissions.updateDependencies ? messages.remove : "kuku",
							label: messages.remove,
							imgSrc: DELETE_ICON,
							isApplicable: dojo.hitch(this, function() {
								return true;
//								return this._workspace.permissions.updateDependencies;
							}),
							handler: function (config, tableData) {
								InfoContentUtils.showQuestionDialog(
									this.messages.dependencydeletetitle,
									this.messages.dependencydelete,
									this.messages.dependencydeleteokbutton,
									this.messages.cancelbutton,
									dojo.hitch(this, function(){
										var args = {};
										args.configurationUri = this._workspaceUri;
										args.dependencyUri = config.uri;
										uri = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + DEPENDENCY_SERVICE+ "/removeDependency?" + dojo.objectToQuery(args);
										var args = {
											url: uri,
											headers: {
												"Accept" : "application/json"
											},
											sync: true,
											handleAs: "json",
											load: function(response, ioArgs) {
												
												// remove table row
												var tr = tableData.parentNode;
												var table = tr.parentNode;
												if (table) {
													table.removeChild(tr);
												}
												
												// remove data entry in configuration resource array
												var configArray = self._configurationResources;
												for (var i = 0; i < configArray.length; i++) {
													var configResource = configArray[i];
													if (configResource === config) {
														self._configurationResources.splice(i, 1);
														break;
													}
												}
											},
											error: dojo.hitch(this, function(e, ioArgs) {
												rmps.core.error.createXhrError(ioArgs).dispatch();
											})
										};
										var deferred = jazz.client.xhrDelete(args);
									})
									)
								}
							}              
						];
			} else if (this._isMutable(configResource)) {
				actions = this._workspaceActions;
			} else {
				actions = this._snapshotActions;
			}
			dojo.forEach(actions, dojo.hitch(this, function (action){
				var button = jazz.ui.toolbar.Button({
					title: action.label,
					imgSrc: action.imgSrc,
					onClick: dojo.hitch(this, function() {
						action.handler.call(this, configResource, td);
					})
				});
				button.setAttribute("enabled", action.isApplicable ? action.isApplicable(configResource) : true);
				toolbar.addChild(button);
			}));
			dojo.addClass(toolbar.domNode, "vvc-workspace-snapshot-toolbar");
			return toolbar;
		},
		
		showMessage: function(args) {
			if (args && this._messageBoxContainer) {
				args["closable"] = true;
				if (!this.messageBox) {
					this.messageBox = new jazz.ui.MessageBox();
					this._messageBoxContainer.appendChild(this.messageBox.domNode);
				}
				this.messageBox.showMessage(args);
				this._messageBoxContainer.style.display = "";
			}
		},
		
		hideMessage: function() {
			if (this._messageBoxContainer) {
				this._messageBoxContainer.style.display = "none";
			}
		},

		getConfigurationResources: function() {
			var uri = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL
			var args = {};
			args.configurationUri = this._workspaceUri;
			if ((this._style == DEPENDENCIES_LIST_STYLE
				|| this._style == DEPENDENCIES_LIST_WITHOUT_ACTIONS_STYLE)
				&& this.workspaceUri != null) {
				// get list of existing dependencies
				// actions in the column have permissions restricted based on
				// editor's configuration, not each dependency's configuration
				uri += DEPENDENCY_SERVICE + "/getExistingDependencies?" + dojo.objectToQuery(args);
			} else if (this._style == PICKER_STYLE) {
				// get list of snapshot can be picked as dependencies
				// there are no actions displayed
				uri +=  DEPENDENCY_SERVICE + "/getSelectableDependencies?" + dojo.objectToQuery(args);
			}
			var args = {
					url: uri,
					headers: {
						"Accept" : "application/json"
					},
					handleAs: "json",
					load: function(response, ioArgs) {
						var configResources = [];
						for (var uri in response) {
							var config = new rmps.app.ui.internal.util.ConfigurationResource();
							config.onLoad(response, uri);
							configResources.push(config);
						}
						return configResources;
					}
			};
			return jazz.client.xhrGet(args);
		}

	})   
})();
