/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.util.ConfigurationResource");
dojo.require("rmps.app.ui.internal.util.DmObject");

dojo.require("dojox.html.entities");
var DecodeHtmlEntity = dojox.html.entities.decode;

dojo.require("rmps.app.ui.internal.util.Constants");
var Constants = rmps.app.ui.internal.util.Constants;

dojo.declare("rmps.app.ui.internal.util.ConfigurationResource", rmps.app.ui.internal.util.DmObject, {
	
	parentUri: null,
	parentName: null,
	title: null,
	permissions: null,
	permissionsLoaded: false,

	getFrontServiceId: function() {
		return "configurations";
	},

	getTitle: function() {
		if (!this.title) {
			this.title = DecodeHtmlEntity(this.getProperty(Constants.TITLE));
		}
		return this.title;
	},
	
	isTitleChanged: function() {
		if (this.title != null) {
			var name = this.getProperty(Constants.TITLE);
			if (this.title != name) {
				return true;
			}
		}
		return false;
	},

	getDescription: function() {
		return this.getProperty(Constants.DESCRIPTION) || "";
	},

	getParentUri: function() {
		if (!this.parentUri) {
			if (this.getMutable() == "true") {
				this.parentUri = this.getParentWorkspaceUri();
			} else {
				this.parentUri = this.getParentSnapshotUri();
			}
		}
		return this.parentUri;
	},
	
	getParentName: function() {
		if (!this.parentName) {
			if (this.getMutable() == "true") {
				this.parentName = this.getParentSnapshotName();
			} else {
				this.parentName = this.getParentWorkspaceName();
			}
		}
		return this.parentName;
	},
	
	getParentWorkspaceUri: function() {
		return this.getProperty("parentWorkspaceUri");
	},

	getParentWorkspaceName: function() {
		return this.getProperty("parentWorkspaceName");
	},
	
	getParentSnapshotUri: function() {
		return this.getProperty("parentSnapshotUri");
	},

	getParentSnapshotName: function() {
		return this.getProperty("parentSnapshotName");
	},
	
	getProjectAreaName: function() {
		return this.getProperty("projectAreaName");
	},
	
	getProjectAreaUri: function() {
		return this.getProperty("projectAreaUri");
	},
		
	getMutable: function() {
		return this.getProperty(Constants.MUTABLE);
	},
	
	isMutable: function() {
		if (this.getMutable() == "true") {
			return true;
		}
	},
	
	isWorkspace: function()
	{
		if (this.getType() == "workspace")
			return true;
		return false;	
	},
	
	isSnapshot: function()
	{
		if (this.getType() == "snapshot")
			return true;
		return false;	
	},
	
	isChangeset: function() {
		if (this.getType() == "changeset")
			return true;
		return false;
	},

	getCreated: function() {
		return this.getProperty(Constants.CREATED);
	},

	getCreator: function() {
		return this.getProperty(Constants.CREATOR);
	},
	
	// check for duplicate name constraint
	isConflictingTitle: function() {
		var newTitle = this.getTitle();
		var oldTitle = this.getProperty(Constants.TITLE);
		if (oldTitle ===  newTitle) {
			// title field has not been changed
			return false;
		}
		
		// TODO: invoke a service to do validation
		return false;
	},
	
	save: function() {
		// load existing configuration
		var config = new rmps.app.ui.internal.util.ConfigurationResource();
		config.load(this.getUri());
				
		// only name and description can be saved
		config.setProperty(Constants.TITLE, this.getTitle());
		config.setProperty(Constants.DESCRIPTION, this.getDescription());
		return config.doSave();
	},

	setTitle: function(value) {
		this.title = value;
	},

	setDescription: function(value) {
		return this.setProperty(Constants.DESCRIPTION, value);
	},
	
	getType: function()
	{
		if (this.getProperty("type") == null)
		{
			var errorMsg = "Assert Failure: Attempting to ask for the configurations type when it's not set on the configuration.";
			console.error(errorMsg);
			alert(errorMsg);
			return;
		}
		return this.getProperty("type");
	},
	
	isArchived: function() {
		//return "http://jazz.net/ns/vvc#Archive" === this.getProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
	},
	
	loadPermissions: function(uri, onError) {
		var args = {
				url: uri,
				headers: {
					"Accept" : "application/json",
				},
				handleAs: "json",
				load: dojo.hitch(this, this.onLoadPermissions),
				error: dojo.hitch(this, this.onError),
				failOk: true
		};
		
		if (onError != null)
			args.error = onError;
		return jazz.client.xhrGet(args);
	},
	
	// (amz) Hack, no way around this in time...
	loadPermissionsSync: function(uri, onError) {
		var args = {
				url: uri,
				headers: {
					"Accept" : "application/json",
				},
				handleAs: "json",
				sync: true,
				load: dojo.hitch(this, this.onLoadPermissions),
				error: dojo.hitch(this, this.onError),
				failOk: true
		};
		
		if (onError != null)
			args.error = onError;
		return jazz.client.xhrGet(args);
	},
		
	onLoadPermissions: function(response, ioArgs) {
		this.permissions = response;
		this.permissionsLoaded = true;
		return this.permissions;
	},
	
	onLoad: function __onLoad(response, uri, ioArgs) {			
		if (ioArgs && ioArgs.xhr) {
			this.eTag = ioArgs.xhr.getResponseHeader("ETag");
		}
		this.uri = uri;
		if (response && response[uri]) {
			this.properties = response[uri];
			if(this.uri){
				this.loaded = true;
				 if (this.properties.permissions) {
					 this.permissions = [];
					 for (var i = 0; i < this.properties.permissions.length; i++) {
					   this.permissions.push(this.properties.permissions[i].value);
					 }
					 this.permissionsLoaded = true;
				}
				
			} else {
				throw "Attempted to load a configuration resource, but the response did not contain a valid, identifiable configuration!";
			}
		}		
		return this;
	}
});
})();
