/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2015, 2016.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function DmButtonFactory() {
dojo.provide("rmps.app.ui.internal.util.DmButtonFactory");

dojo.require("jazz.ui.toolbar.Button");
dojo.require("rmps.core.RootServices");

dojo.require("jazz.ui.menu.MenuItem");
var MenuItem = jazz.ui.menu.MenuItem;

dojo.require("com.ibm.team.repository.web.configurationManagement._utils.CmCompareDeliverUtils");
var CmCompareDeliverUtils = com.ibm.team.repository.web.configurationManagement._utils.CmCompareDeliverUtils;

dojo.require("rmps.ui.util.UIUtil");
var UIUtil = rmps.ui.util.UIUtil;

dojo.require("rmps.app.ui.internal.util.UIUtils");
var UIUtils = rmps.app.ui.internal.util.UIUtils;

dojo.require("com.ibm.team.configuration.web.context.ContextPicker");
var ContextPicker = com.ibm.team.configuration.web.context.ContextPicker;
var ContextPickerConfigurationFilterTypes = com.ibm.team.configuration.web.context.ContextPickerConfigurationFilterTypes;

dojo.require("rmps.app.ui.internal.util.InfoContentUtils");
var InfoContentUtils = rmps.app.ui.internal.util.InfoContentUtils;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.LinkToWorkItemUtil");
var LinkToWorkItemUtil = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.LinkToWorkItemUtil;

dojo.require("com.ibm.team.repository.web.configurationManagement.notifications.CmNotificationFactory");
var CmNotificationFactory = com.ibm.team.repository.web.configurationManagement.notifications.CmNotificationFactory;

dojo.requireLocalization("rmps.app.ui.internal.pages", "DmChangeSetEditorMessages");
var DmChangeSetEditorMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DmChangeSetEditorMessages");

dojo.requireLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.context", "ConfigurationContextMessages");
var configurationContextMessages = dojo.i18n.getLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.context", "ConfigurationContextMessages");

dojo.requireLocalization("rmps.app.ui.internal.util", "DmButtonFactoryMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.util", "DmButtonFactoryMessages");

dojo.requireLocalization("com.ibm.team.repository.web.configurationManagement", "CmButtonFactoryMessages");
var CmButtonFactoryMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.configurationManagement", "CmButtonFactoryMessages");

rmps.app.ui.internal.util.DmButtonFactory = {
	
	getLinkToWorkItemButton: function (data, addLinkCallback, isMenuItem) {
		var args = {
				imgClass: "icons-changeset-workitem",
				title: DmChangeSetEditorMessages.addLink,
				onClick: dojo.hitch(this, function() {
					 LinkToWorkItemUtil.linkToWorkItemAction(data, addLinkCallback);
				})
			};
		var applicable = function(button){
			return true;
		};
		return isMenuItem ? this._makeMenuItem(args, null, applicable) : this._makeButton(args, null, applicable);
	},
	
	getReverseChangesetButton: function getReverseChangesetButton(data, options, onClickArgs){
		var args = {
			imgClass: "icons-changeset-reverse",
			title: CmButtonFactoryMessages.reverseChangeset,
			onClick: dojo.hitch(this, function(){							
				InfoContentUtils.showQuestionDialog(
					CmButtonFactoryMessages.reverseChangesetTitle,
					dojo.string.substitute(CmButtonFactoryMessages.reverseChangesetContent, {changesetTitle: data.name}),
					CmButtonFactoryMessages.reverseChangesetOkButton,
					CmButtonFactoryMessages.cancelButton,
					dojo.hitch(this, function(){
						var params = {
							reverse : true,
							streamUri : data.parentStreamUri,
							changesetUri : data.id,
							changesetTitle : data.name 
						};
						var def = jazz.client.xhrPost({
							handleAs: 'text',
							headers: {
								"Content-Type" : "text/plain"
							},
							url: rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "dm-changesets") + "?" + dojo.objectToQuery(params)
						});
						
						var tempContext = {
							closeEvent: dojo.hitch(this, function(){
								if(options && options.actionCallback){
									options.actionCallback();
								}
							})
						};
						var notification = CmNotificationFactory.createInfo({
							title: CmButtonFactoryMessages.initializingReverseChangeset,
							closeOnEvent: {
								context: tempContext,
								event: "closeEvent"
							}
						}).dispatch();
						
						def.then(dojo.hitch(this, function(results){
							tempContext.closeEvent();
						}), dojo.hitch(this, function(error){
							notification.close();
							CmNotificationFactory.createXhrError({xhrError: error, ioArgs: error.response.options.ioArgs}).dispatch();
						}));
					})
				);
			})
		};
		
		var applicable = function(button){
			return "committed" === data.state.toLowerCase();
		};
		
		var button = this._makeMenuItem(args, options, applicable);
		return button;
	},
	
	getCompareWithBaseSnapshotButton: function __getCompareWithBaseSnapshotButton(data, options, onClickArgs){
		var args = {
			imgClass: "icons-compare-with-base-snapshot",
			title: (options && options.title) ? options.title : CmButtonFactoryMessages.compareWithBaseSnapshotButtonTitle,
			onClick: dojo.hitch(this, function __getCompareWithBaseSnapshotButtonOnClick(){
				UIUtil.showLoadingMessage();

				// onClickArgs are required!
				var args = data ? data : {};
				if(onClickArgs){
					// onClick args override create-time args
					dojo.mixin(args, onClickArgs());
				}
				
				var startDeliver = CmCompareDeliverUtils.startCompare(args).then(function __onStartCompareOk(){
					UIUtil.hideLoadingMessage();
				}, function __onStartCompareError(error){
					UIUtil.hideLoadingMessage();
					CmNotificationFactory.createXhrError({xhrError: error, ioArgs: args}).dispatch();
				});
			})
		};
		var applicable = function __getCompareWithBaseSnapshotButtonApplicable(button){
			if (data) {
				return !data.archived;
			}
			return true; 
		};
		return this._makeButton(args, options, applicable);
	},
	
	getCompareButton: function __getCompareButton(data, options, onClickArgs){
		var args = {
			imgClass: options.imgClass,
			title: options.title,
			onClick: dojo.hitch(this, function __getCompareButtonOnClick(){
				var picker = new ContextPicker({
					filterObject: {
						validTypes: [ContextPickerConfigurationFilterTypes.streams]
					},
					projectAreaUrl : ""
				});
				picker.onOk = dojo.hitch(this, function(configuration) {
					// onClickArgs are required!
					var args = data ? data : {};
					if(onClickArgs){
						// onClick args override create-time args
						dojo.mixin(args, onClickArgs());
					}
					if (args.sourceUri === configuration.uri) {
						InfoContentUtils.showSimpleContentDialog(messages.configurationSelectFailedTitle, messages.errSrcTrgAreEqual);
						return;
					}
					if (configuration.rdfType === "changeSet") {
						InfoContentUtils.showSimpleContentDialog(messages.configurationSelectFailedTitle, messages.errTargetIsNotStreamOrBaseline);
						return;
					}
					args.targetUri = configuration.uri;
					
					var startDeliver = CmCompareDeliverUtils.startCompare(args).then(function __onStartCompareOk(){
					}, function __onStartCompareError(error){
						CmNotificationFactory.createXhrError({xhrError: error, ioArgs: args}).dispatch();
					});
				});

			})
		};
		var applicable = function __getCompareButtonApplicable(button){
			if (data) {
				return !data.archived;
			}
			return true;
		};
		return this._makeButton(args, options, applicable);
	},
	
	getAcceptButton: function __getAcceptButton(data, options, onClickArgs){
		var args = {
			imgClass: options.imgClass,
			title: options.title,
			onClick: dojo.hitch(this, this.onClickAccept, data, onClickArgs)
		};
		var applicable = function __getAcceptButtonApplicable(button){
			if (data) {
				return !data.archived;
			}
			return true; 
		};
		return this._makeButton(args, options, applicable);
	},
	
	onClickAccept: function(data, onClickArgs) {
		var picker = new ContextPicker({
			filterObject: {
				validTypes: [ContextPickerConfigurationFilterTypes.streams]
			},
			projectAreaUrl : ""
		});
		picker.onOk = dojo.hitch(this, function(configuration) {
			// onClickArgs are required!
			var args = data ? data : {};
			if(onClickArgs){
				// onClick args override create-time args
				dojo.mixin(args, onClickArgs());
			}
			if (args.targetUri === configuration.uri) {
				InfoContentUtils.showSimpleContentDialog(messages.configurationSelectFailedTitle, messages.errSrcTrgAreEqual);
				return;
			}
			if (configuration.rdfType === "changeSet") {
				InfoContentUtils.showSimpleContentDialog(messages.configurationSelectFailedTitle, messages.errTargetIsNotStreamOrBaseline);
				return;
			}
			args.sourceUri = configuration.uri;
			
			var startDeliver = CmCompareDeliverUtils.startDeliver(args).then(function __onStartAcceptOk(){
			}, function __onStartAcceptError(error){
				CmNotificationFactory.createXhrError({xhrError: error, ioArgs: args}).dispatch();
			});
		});

	},
	
	onClickDeliver: function(data, onClickArgs) {
		var picker = new ContextPicker({
			filterObject: {
				validTypes: [ContextPickerConfigurationFilterTypes.streams]
			},
			projectAreaUrl : ""
		});
		picker.onOk = dojo.hitch(this, function(configuration) {
			// onClickArgs are required!
			var args = data ? data : {};
			if(onClickArgs){
				// onClick args override create-time args
				dojo.mixin(args, onClickArgs());
			}
			if (args.sourceUri === configuration.uri) {
				InfoContentUtils.showSimpleContentDialog(messages.configurationSelectFailedTitle, messages.errSrcTrgAreEqual);
				return;
			}
			if (configuration.rdfType === "changeSet") {
				InfoContentUtils.showSimpleContentDialog(messages.configurationSelectFailedTitle, messages.errTargetIsNotStreamOrBaseline);
				return;
			}
			args.targetUri = configuration.uri;
			
			var startDeliver = CmCompareDeliverUtils.startDeliver(args).then(function __onStartDeliverOk(){
			}, function __onStartDeliverError(error){
				CmNotificationFactory.createXhrError({xhrError: error, ioArgs: args}).dispatch();
			});
		});

	},
	
	getChangeSetDeliverButton: function __ggetChangeSetDeliverButton(data, options, onClickArgs){
		var args = {
			imgClass: "icons-deliver",
			title: configurationContextMessages["commitChangeset"],
			onClick: dojo.hitch(this, function __getDeliverButtonOnClick(){
				var waitingDialog = InfoContentUtils.showWaitingDialog();

				// onClickArgs are required!
				var args = data ? data : {};
				
				if(onClickArgs){
					// onClick args override create-time args
					dojo.mixin(args, onClickArgs());
				}
				
				var startDeliver = CmCompareDeliverUtils.startDeliver(args).then(function __onStartCompareOk(){
					waitingDialog.close();
				}, function __onStartCompareError(error){
					waitingDialog.close();
					var ncArgs = {
							title: configurationContextMessages["failedToCommitChangesetWithoutName"],
							description: configurationContextMessages["failedToCommitChangeset.explanation"],
							solution: configurationContextMessages["failedToCommitChangeset.useraction"]
						}
					ncArgs.xhrError = error;
					ncArgs.ioArgs = args;
					CmNotificationFactory.createXhrError(ncArgs).dispatch();
				});
			})
		};
		var applicable = function __getDeliverButtonApplicable(button){
			if (data) {
				return !data.archived;
			}
			return true;
		};
		var button = this._makeButton(args, options, applicable);
		return button;
	},
	
	getDeliverButton: function __getDeliverButton(data, options, onClickArgs){
		var args = {
			imgClass: options.imgClass,
			title: options.title,
			onClick: dojo.hitch(this, this.onClickDeliver, data, onClickArgs)
		};
		var applicable = function __getDeliverButtonApplicable(button){
			if (data) {
				return !data.archived;
			}
			return true;
		};
		return this._makeButton(args, options, applicable);
	},
	
	
	//Helper functions coppied from com.ibm.team.repository.web.configurationManagement.CmButtonFactory
	_makeButton: function (args, options, applicable){
		// Force enabled to be false, wait for the isApplicable code to run
		args.enabled = false;
		if(options && options.titleAsLabel){
			args.label = args.title;
		}
		var button = new jazz.ui.toolbar.Button(args);
		var isApplicable = dojo.hitch(this, applicable, button);
		dojo.when(isApplicable(), dojo.hitch(this, function(isEnabled){
			// Respect the explicit disablement of a button, then respect the buttons own enablement.
			button.set("enabled", options && options.disabled !== undefined ? options.disabled === false : true && isEnabled);
		}));
		return button;
	},
		
	_makeMenuItem: function (args, options, applicable){
		// Make sure the label exists
		args.disabled = true;
		if(args.title && !args.label){
			args.label = args.title;
		}
		// Map the Button 'imgClass' to the Menu Item 'iconClass'
		if(args.imgClass && !args.iconClass){
			args.iconClass = args.imgClass + " vvcIcon";
		}
		
		var menuItem = new MenuItem(args);
		
		var isApplicable = dojo.hitch(this, applicable, menuItem);
		dojo.when(isApplicable(), dojo.hitch(this, function(isEnabled){
			// Respect the explicit disablement of a button, then respect the buttons own enablement.
			var enabled = options && options.disabled !== undefined ? options.disabled === false : true && isEnabled;
			menuItem.set("disabled", !enabled);
		}));
		return menuItem;
	}
};
}());