/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function DmButtonFactory() {
dojo.provide("rmps.app.ui.internal.util.DmTabsBuilder");

dojo.require("jazz.ui.toolbar.Button");
var Button = jazz.ui.toolbar.Button;

dojo.require("rmps.app.ui.internal.pages.WorkspacesAndSnapshotsViewer");
var WorkspacesAndSnapshotsViewer = rmps.app.ui.internal.pages.WorkspacesAndSnapshotsViewer;

dojo.require("rmps.app.ui.internal.util.ConfigurationResource");
var ConfigurationResource = rmps.app.ui.internal.util.ConfigurationResource;

dojo.require("com.ibm.team.repository.web.configurationManagement.editors.CmTabManager");
var Tab = com.ibm.team.repository.web.configurationManagement.editors.CmTab;

dojo.requireLocalization("rmps.app.ui.internal.pages", "DmStreamEditorMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DmStreamEditorMessages");

dojo.require("rmps.app.ui.internal.util.Constants");
var Constants = rmps.app.ui.internal.util.Constants;

dojo.require("com.ibm.team.configuration.web.context.ContextPicker");
var ContextPicker = com.ibm.team.configuration.web.context.ContextPicker;
var ContextPickerConfigurationFilterTypes = com.ibm.team.configuration.web.context.ContextPickerConfigurationFilterTypes;

var DEPENDENCY_SERVICE = "/dependencies";

rmps.app.ui.internal.util.DmTabsBuilder = {
	
	buildDependenciesTab: function (editor, canModify) {
		var initialiser = dojo.hitch(this, function(){
			// (amz) This whole thing is a 'tactical fix'
			var workspaceResource = new ConfigurationResource();
			workspaceResource.load(editor.resourceUri);
			
			var containerNode = dojo.create("div",{
				style: "width: 99%;"
			});
			
			var actionsNode = dojo.create("div", {
				"class": "dependencyActions"
			},containerNode);
			
			if (canModify) {
				var addDependencyButton = new Button({
					title: messages.addDependency,
					imgClass: "addDependencyButtonIcon",
					onClick: dojo.hitch(this, function __onClickAddDependencyButton(){
						
						var picker = new ContextPicker({
							filterObject: {
								validTypes: [ContextPickerConfigurationFilterTypes.baselines]
							},
							projectAreaUrl: Constants.NONE
						});
						picker.onOk = dojo.hitch(this, function(configuration) {
							this._addSelected(editor, configuration.uri);
						});
					})
				});
				actionsNode.appendChild(addDependencyButton.domNode);
			}
			
			editor.workspacesViewer = new WorkspacesAndSnapshotsViewer({
				workspace: workspaceResource,
				workspaceUri: editor.resourceUri,
				style: canModify ? 2 : 3, 
				lazyStart: false
			});
			containerNode.appendChild(editor.workspacesViewer.domNode);
			
			return containerNode;
		});
		
		var tab = new Tab({
			id: "dependencies",
			title: messages.dependenciesTabTitle,
			index: 1,
			initialiser: initialiser,
			helpContent: messages.dependenciesHelpContent
		});
		return tab;
	},
	
	_addSelected: function(editor, dependencyUri) {
		if (dependencyUri) {
			editor.workspacesViewer.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.adding
			});
			
			var args = {};
			args.configurationUri = editor.resourceUri;
			args.dependencyUri = dependencyUri;
			uri = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + DEPENDENCY_SERVICE + "/addDependency?" + dojo.objectToQuery(args);
			var self = editor;
			var args = {
				url: uri,
				headers: {
					"Accept" : "application/json"
				},
				sync: true,
				handleAs: "json",
				error: dojo.hitch(this, function(e, ioArgs) {
					editor.workspacesViewer.hideMessage();
					rmps.core.error.createXhrError(ioArgs).dispatch();
				})
			};
			var deferred = jazz.client.xhrPost(args);
			deferred.addCallback(this, function (param) {
				editor.workspacesViewer.hideMessage();
				editor.workspacesViewer.refresh();
			});
		} else {
			editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.addEmptySelection
			});
		}
	}
};
}());